const express = require('express');
const bodyParser = require('body-parser');  
const nodemailer = require('nodemailer');
const { sshTunnelConfig, dbServer, emailConfig } = require('./../config');
const createSshTunnel = require('./../sshTunnel');
const queryDatabase = require('./../databasequery');
const cors = require('cors');   

const app = express();
const PORT = 4500;

// Middleware to handle CORS and preflight requests
app.use((req, res, next) => {
    res.append('Access-Control-Allow-Origin', '*');
    res.append('Access-Control-Allow-Methods', 'GET,HEAD,PUT,PATCH,POST,DELETE,OPTIONS');
    res.append('Access-Control-Allow-Headers', 'Content-Type, authorization');
    next();
});

app.use(bodyParser.json());  
app.use(cors());    

// Define routes for App1 
app.get('/', (req, res) => { 
	res.send('Common member frontend');    		
});  

// Nodemailer setup
const transporter = nodemailer.createTransport({
  service: emailConfig.service,
  auth: {
    user: emailConfig.user, 
    pass: emailConfig.pass,
  }
});

// Send verification email
function sendVerificationEmail(email_address) {
  const mailOptions = {
    from: emailConfig.user, // Replace with your Gmail address
    to: email_address,
    subject: 'Verify Your Email',
    html: `<p>Click the link to verify your email:</p>
    <p><a href="http://157.245.36.128:5200/verify/${email_address}" class="btn" style="background: #e33439; color: #fff; padding: 10px 20px; font-size: 16px; border-radius: 7px;">Verify Email</a></p>`
  };

  transporter.sendMail(mailOptions, (error, info) => {
    if (error) {
      console.error(error);
    } else {
      console.log('Email sent: ' + info.response);
    }
  });
}

// forgot email
function sendForgotEmail(email, new_pwd) {
	const mailOptions = {
	  from: emailConfig.user,
	  to: email,
	  subject: 'New Password for membership Login',
	  html: `<!DOCTYPE html>
	  <html>
	  <head>
		<meta name='viewport' content='width=device-width' />
		<title>Danshop</title>
		<link href='https://fonts.googleapis.com/css2?family=Lato:wght@100;300;400;700;900&display=swap'  rel='stylesheet'>
		<style>  body { padding-top: 0 !important; padding-bottom: 0 !important; padding-top: 0 !important; padding-bottom: 0 !important; margin: 0 !important; width: 100% !important; -webkit-text-size-adjust: 100% !important; -ms-text-size-adjust: 100% !important; -webkit-font-smoothing: antialiased !important; font-size: 14px; line-height: 22px; font-family: 'Lato', sans-serif; font-weight: 400; }
		</style>
	  </head>
	  
	  <body paddingwidth='0' paddingheight='0' style='' offset='0' toppadding='0' leftpadding='0'>
		<div style='display:table; width:600px !important; margin: 0 auto; background: #fff; padding:20px;'>
		  <table width='600' border='0' cellspacing='0' cellpadding='0' class='tableContent bgBody' align='center' style='width: 600px; display: block; background: #f3f3f3'>
			<tbody>
			  <tr>
				<table class='MainContainer' width='600' cellspacing='0' cellpadding='0' border='0' bgcolor='#fbfbfb'
				  align='center'
				  style='width: 600px; display: block; padding: 20px;background-image: url(http://157.245.36.128:5200/assets/images/border.jpg);background-repeat: no-repeat;background-position: right top;background-attachment: scroll;background-size: 20px auto;'>
				  <tbody style='width: 560px; display: block; background: #f3f3f3;'>
					<tr>
					  <td>
						<div style='margin:20px 0 15px 30px' class='contentLogo'><a href='http://157.245.36.128:5200' target="_blank"><img src='http://157.245.36.128:5200/assets/images/red-logo.png' width='200px' alt='' border='0'></a>
						</div>
					  </td>
					</tr>
					<tr>
					  <td>
						<div style='padding: 0; margin: 30px; background: #eaeaea;-webkit-border-radius: 10px;-moz-border-radius: 10px;border-radius: 10px;' class='contentEditTable'>
						  <div style='padding: 15px 0 5px;' class='box-step'>
							<div style='float: left; width: 100%;' class='box-left'>
							  <div style='padding: 15px 20px 0 30px; color:#606060; position:relative;'  class='box-padd'>
								<p><strong>Your new password has been created. Keep it safe and secure</strong></p>
                <p> ${new_pwd}</p>
							  </div>
							</div>
							<div style='clear: both;'></div>
						  </div>
						</div>
					  </td>
					</tr>
	  
					<tr>      
					  <td>
						<div style='padding: 0 30px 35px; color:#606060; text-align:center;' class='contentTitle h3-mrg'>
						  <p style='margin:0 0 15px !important; font-size: 16px;'>Missed out on something? Fret not and click the button to know more about the products and business</p>
						  <p style='margin:0 0 30px !important; font-size: 16px;'>
							<a href='#' style='display: inline-block;background: #dfb15d;color: #fff;padding: 11px 25px;-webkit-border-radius: 10px;-moz-border-radius: 10px;border-radius: 10px;text-decoration: none;font-size: 18px;'>Let’s Connect</a>
						  </p>
	  
						  <p style='margin:0 0 15px !important; font-size: 16px;'>
							<a target="_blank" style='display: inline-block;padding: 0 10px;' href="https://www.facebook.com/DANSA.org/"> <img src="http://157.245.36.128:5200/assets/images/facebook.png" width="40" height="40"></a>
							<a target="_blank" href="https://twitter.com/divesafety"></a>
							<a target="_blank" href="https://www.pinterest.com/dansa_org/"></a>
							<a target="_blank" style='display: inline-block;padding: 0 10px;' href="https://www.instagram.com/dansouthernafrica/"><img src="http://157.245.36.128:5200/assets/images/instagram.png" width="40" height="40"></a>
							<a target="_blank" style='display: inline-block;padding: 0 10px;' href="https://www.youtube.com/c/DANSouthernAfrica"><img src="http://157.245.36.128:5200/assets/images/youtube.png" width="40" height="40"></a>
							<a target="_blank" href="https://www.linkedin.com/company/divers-alert-network-southern-africa"></a>
						  </p>
						  <p style='margin:0 !important; font-size: 16px;'><a href='http://157.245.36.128:5200/'
							  style='color: #606060;text-decoration: none;letter-spacing:0.5px;'>www.DANSHOP.com</a></p>
						</div>
					  </td>
					</tr>
				  </tbody>
				</table>
			  </tr>
			</tbody>
		  </table>
		</div>
	  </body>
	  </html>`
	};
  
	transporter.sendMail(mailOptions, (error, info) => {
	  if (error) {
		console.error(error);
	  } else {
		console.log('Email sent: ' + info.response);
	  }
	});
}

// register save data to membership table
app.post('/save', async (req, res) => {            
  const data = req.body;                                           

  try {
    // Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    const randomNo = Math.floor(Math.random() * 1000); // Generate a random number
    
    // Insert data into the database  
    const title=(typeof data.title=="undefined")?'':data.title;  	
    const first_name=(typeof data.first_name=="undefined")?'':data.first_name;    
    const last_name=(typeof data.last_name=="undefined")?'':data.last_name;  	
    const dob=(typeof data.dob=="undefined")?'':data.dob;  
    const passport_no=(typeof data.passport_no=="undefined")?'':data.passport_no;  		
    const address1=(typeof data.address1=="undefined")?'':data.address1;  	
    const address2=(typeof data.address2=="undefined")?'':data.address2;  	
    const city=(typeof data.city=="undefined")?'':data.city;  	
    const state=(typeof data.state=="undefined")?'':data.state;   	
    const postal_code=(typeof data.postal_code=="undefined")?'':data.postal_code;  	
    const country=(typeof data.country=="undefined")?'':data.country;    
    const tel_home=(typeof data.tel_home=="undefined")?'':data.tel_home;  	
    const tel_work=(typeof data.tel_work=="undefined")?'':data.tel_work;  	
    const cell_phone=(typeof data.cell_phone=="undefined")?'':data.cell_phone;  	
    const email_address=(typeof data.email_address=="undefined")?'':data.email_address;  	
    const fax=(typeof data.fax=="undefined")?'':data.fax;  
    const aid=(typeof data.aid=="undefined")?'':data.aid;  	 	
    const aid_no=(typeof data.aid_no=="undefined")?'':data.aid_no;  	 	
    const new_pwd=(typeof data.new_pwd=="undefined")?'':data.new_pwd;  
    const start_date=(typeof data.start_date=="undefined" || data.start_date== null)?'0000:00:00':data.start_date;   
    const package_name=(typeof data.package_name=="undefined")?'':data.package_name;  	 	
    const package_price=(typeof data.package_price=="undefined")?'':data.package_price; 
    const package_for=(typeof data.package_for=="undefined")?'':data.package_for;  	 	
    const choose_membership=(typeof data.choose_membership=="undefined")?'':data.choose_membership;  	 	  

    let memberPrefix = ''; // Initialize member prefix
    // Check package_name to determine the member prefix
    if(choose_membership === 'annual'){
      if (package_name === 'Standard') {
          memberPrefix = 'DANS';
      } else if (package_name === 'Plus') {
          memberPrefix = 'DANP';
      } else if (package_name === 'Master Dive Pro') {
          memberPrefix = 'DANM';
      }else if (package_name === 'Master Tech') {
        memberPrefix = 'DANX';
      }
    }else if(choose_membership === 'student'){
      memberPrefix = `DANT`;
    }
    else if(choose_membership === 'temporary'){
      memberPrefix = `DANE`;
    }
    else if(choose_membership === 'commercial'){
      if (package_name === 'individual') {
        memberPrefix = 'DANCI';
      } else if (package_name === 'professional') {
          memberPrefix = 'DANCP';
      } else if (package_name === 'company') {
          memberPrefix = 'DANCC';
      }else if (package_name === 'student') {
        memberPrefix = 'DANCS';
      }
    }
    
    const memberNO = `${memberPrefix} ${randomNo}`; // Generate random number and the determined prefix

    // Check if the email already exists in the database
    const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
    const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${email_address}'`;
    const existingUser = await queryDatabase(emailCheckQuery, emailStream);

    if (existingUser.length > 0) {
      return res.status(500).json({ message: 'Email already exists' });
    }
    else{
      const sqlQuery = `INSERT INTO membership (title1, fname1, sname1, dob1, passport1, address1,address2,city1,province1,postcode1,country1,telephone1,telephone2,cellphone1,fax,email1,password, medical_aid,medical_aid_no,member_no, package_name,package_price,package_for,verifyEmail,verifyAmount,choose_membership,status1,startdate,joindate)  VALUES ('${title}','${first_name}','${last_name}','${dob}','${passport_no}', '${address1}','${address2}','${city}','${state}','${postal_code}','${country}','${tel_home}','${tel_work}','${cell_phone}','${fax}','${email_address}','${new_pwd}', '${aid}','${aid_no}', '${memberNO}', '${package_name}','${package_price}', '${package_for}', 'not_verified','not_paid', '${choose_membership}', 'INACTIVE', '${start_date}', '${start_date}')`;
      
      await queryDatabase(sqlQuery, stream);

      // Send a verification email
      sendVerificationEmail(email_address);

      // Send a success response
      res.status(200).json({ message: 'Members registered successfully. Check your email for verification.' });
    }
	 
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error });
  } 
});

// API endpoint to verify email
app.get('/verify/:email_address', async (req, res) => {
  const email_address = req.params.email_address;

  // Check current status before updating
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    
    const checkStatusQuery = `SELECT verifyEmail FROM membership WHERE email1 = '${email_address}'`;
    const statusResult = await queryDatabase(checkStatusQuery, stream);

    if (statusResult.length === 0) {
      res.status(404).json({ error: 'Email not found in database.' });
      return;
    }

    const currentStatus = statusResult[0].verifyEmail;
    // console.log(currentStatus);

    if (currentStatus === 'verified') {
      res.status(400).json({ error: 'Email is already verified.' });
      return;
    }

    // Update status to 'verified'
    const streamTwo = await createSshTunnel(sshTunnelConfig, dbServer);
    const updateQuery = `UPDATE membership SET verifyEmail = 'verified' WHERE email1 = '${email_address}'`;
    const updateResult = await queryDatabase(updateQuery, streamTwo);

    res.status(200).json({ message: 'Email verified successfully. You can now log in.' });
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });
  }
});


// resend-email
app.post('/resend', async (req, res) => {
  const data = req.body;  
  
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    const email_address=(typeof data.email=="undefined")?'':data.email;  	

    // Check if the email exists in the database
    const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${email_address}'`;
    const user = await queryDatabase(emailCheckQuery, stream);

    if (user.length === 0) {
      return res.status(404).json({ message: 'Email not found' });
    }

    // Resend the verification email
    sendVerificationEmail(email_address);

    // Send a success response
    res.status(200).json({ message: 'Verification email resent successfully' });

  } catch (error) {
    console.error('Error resending verification email:', error);
    res.status(500).json({ error: 'Internal server error' });
  }
});

// if email already exist in membership table on registeration
app.get('/checkEmail/:email_address', async (req, res) => {
  const email_address = req.params.email_address;

  try {
      // Check if the email exists in the database
      const stream = await createSshTunnel(sshTunnelConfig, dbServer);
      const checkEmailQuery = `SELECT * FROM membership WHERE email1 = '${email_address}'`;
      const emailResult = await queryDatabase(checkEmailQuery, stream);

      // If email exists in the database
      if (emailResult.length > 0) {
          res.status(200).json({ exists: true });
      } else {
          res.status(200).json({ exists: false });
      }
  } catch (error) {
      res.status(500).json({ error: 'Error fetching data from database: ' + error.message });
  }
});


// login to dashboard
app.post('/login_to_dashboard', async (req, res) => {            
  const data = req.body; 
  const email = data.email || '';
  const password = data.password || '';    

  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);

    const sqlQuery = `SELECT * FROM membership WHERE email1 = '${email}' AND password = '${password}'`;
    
    // Pass stream as an additional parameter to queryDatabase
    const userData = await queryDatabase(sqlQuery, stream);

    if (userData.length > 0) {    
      const { id, verifyEmail } = userData[0];

      if (verifyEmail === 'verified') {
        // Authentication successful (matching record found and email verified)
        res.status(200).json({ message: 'Member login success. Please wait...', data: userData });
      } else {
        // Email not verified
        res.status(401).json({ error: 'Email is not verified. Please verify your email address.' });
      }   
    } else {
      // Authentication failed (no matching record found)
      res.status(400).json({ error: 'Invalid credentials' });
    }
  } catch (error) {
    console.error('Error querying database:', error);
    res.status(500).json({ message: 'Internal server error', error:error.message });
  }                             
});

// forgot password
function generateRandomPassword(length) {
  const charset = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"; 
  let password = "";
  for (let i = 0; i < length; i++) {
      const randomIndex = Math.floor(Math.random() * charset.length);
      password += charset[randomIndex];
  }
  return password;
}

app.post('/forgot', async (req, res) => {            
  const data = req.body; 
  const email = data.email2 || ''; 
  const new_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6);

  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    const sqlQuery = `SELECT * FROM membership WHERE email1 = '${email}'`;
    const userData = await queryDatabase(sqlQuery, stream);

    if (userData.length > 0) {    
      const { id, verifyEmail } = userData[0];

      if (verifyEmail === 'verified') {
        const stream2 = await createSshTunnel(sshTunnelConfig, dbServer);
        const sqlQuery2 = `UPDATE membership SET password='${new_pwd}' WHERE id=${id}`;   				
		    await queryDatabase(sqlQuery2,stream2);  
        res.status(200).json({ message: 'New Password will be sent to your email address.'});
        sendForgotEmail(email, new_pwd);
      } else {
        // Email not verified
        res.status(401).json({ error: 'Email is not verified. Please verify your email address.' });
      }   
    } else {
      // Authentication failed (no matching record found)
      res.status(400).json({ error: 'Email is not found in our records.' });
    }
  } catch (error) {
    console.error('Error querying database:', error);
    res.status(500).json({ message: 'Internal server error', error:error.message });
  }                             
});


// payment paynow for student 
app.post('/paynow_stud', async (req, res) => {		    	
  const data = req.body;	 
  const stream = await createSshTunnel(sshTunnelConfig, dbServer);	    
  const sqlQuery = 'SELECT (IFNULL(MAX(id),0)+1) AS max_id FROM membership';	  	 	  
  const result = await queryDatabase(sqlQuery, stream);	   
  const next_id = (result.length>0)?result[0].max_id:0;      
    
  if (!data.title || data.title === "") {
    return res.status(500).json({ error: 'Title is required','error_field':'title' });
  }
  else if (!data.fname || data.fname === "") {
    return res.status(500).json({ error: 'First Name is required','error_field':'fname' });
  }
  else if (!data.sname || data.sname === "") {
    return res.status(500).json({ error: 'Surname is required','error_field':'sname' });    
  } else if (!data.dob || data.dob === "") {
    return res.status(500).json({ error: 'Date of birth is required','error_field':'dob' });		
  } 
  else if (!data.address1 || data.address1 === "") {
    return res.status(500).json({ error: 'Postal address is required','error_field':'address1' });		
  } 
  else if (!data.city || data.city === "") {
    return res.status(500).json({ error: 'City is required','error_field':'city' });		
  }
  else if (!data.province || data.province === "") {
    return res.status(500).json({ error: 'State/Province is required','error_field':'province' });	  	
  }
  else if (!data.post || data.post === "") {
    return res.status(500).json({ error: 'Postal Code is required','error_field':'post' });		
  }
  else if (!data.country || data.country === "") {
    return res.status(500).json({ error: 'Country is required','error_field':'country' });		
  }   
  else if (!data.cellphone || data.cellphone === "") {
    return res.status(500).json({ error: 'Cell phone is required','error_field':'cellphone' });		
  } else if (!data.email || data.email === "") {
    return res.status(500).json({ error: 'Email is required','error_field':'email' });		
  }else if (!data.aid || data.aid === "") {
    return res.status(500).json({ error: 'Medical aid is required','error_field':'aid' });
  }
  else if (!data.aid_no || data.aid_no === "") {
    return res.status(500).json({ error: 'Medical aid no. is required','error_field':'aid_no' });		
  }
  else if (!data.start_date || data.start_date === "") {
    return res.status(500).json({ error: 'Course start date is required','error_field':'start_date' });		
  }  
  else if (!data.user_id || data.user_id === "") {
    return res.status(500).json({ error: 'Id is Missing','error_field':'user_id' });		
  } 
  else if (!data.user_type || data.user_type === "") {
    return res.status(500).json({ error: 'Type is Missing','error_field':'user_type' });		
  }  
  else if (!data.pay_type || data.pay_type === "") {  
    return res.status(500).json({ error: 'Payment Type is Missing','error_field':'pay_type' });       
  } 
  else    
  {  	  
  
  try {	 		  

		const id=(typeof data.user_id=="undefined")?0:data.user_id;  
		const type=(typeof data.user_type=="undefined")?'':data.user_type.toUpperCase();   	
		const payType=(typeof data.pay_type=="undefined")?'':data.pay_type;   
		const memberNO = `DANT ${next_id}`;       	

		const instructor_id=(typeof data.user_id=="undefined")?'':data.user_id;         
		const studAnnual_title=(typeof data.title=="undefined")?'':data.title;  	
		const studAnnual_fname=(typeof data.fname=="undefined")?'':data.fname;    
		const studAnnual_sname=(typeof data.sname=="undefined")?'':data.sname;  
		const studAnnual_dob=(typeof data.dob=="undefined" || data.dob== null)?'0000:00:00':data.dob;    	
		const studAnnual_passport=(typeof data.passport=="undefined")?'':data.passport;  
		const studAnnual_address1=(typeof data.address1=="undefined")?'':data.address1;  	
		const studAnnual_address2=(typeof data.address2=="undefined")?'':data.address2;  	
		const studAnnual_city=(typeof data.city=="undefined")?'':data.city;  	
		const studAnnual_province=(typeof data.province=="undefined")?'':data.province;  	
		const studAnnual_post=(typeof data.post=="undefined")?'':data.post;  	
		const studAnnual_country=(typeof data.country=="undefined")?'':data.country; 
		const studAnnual_home=(typeof data.phonehome=="undefined")?'':data.phonehome;  	    	
		const studAnnual_cellphone=(typeof data.cellphone=="undefined")?'':data.cellphone;         
		const studAnnual_email=(typeof data.email=="undefined")?'':data.email;        	
		const studAnnual_aid=(typeof data.aid=="undefined")?'':data.aid;    
		const studAnnual_aid_no=(typeof data.aid_no=="undefined")?'':data.aid_no;     	  
		const studAnnual_course=(typeof data.start_date=="undefined" || data.start_date== null)?'0000:00:00':data.start_date;   

		const stream = await createSshTunnel(sshTunnelConfig, dbServer);	    
	  const sqlQuery = `SELECT * FROM membership where id='${instructor_id}'`;	  	 	  
	  const result = await queryDatabase(sqlQuery, stream);	   
		
	  const pack_name = (result.length>0)?result[0].package_name:'';    
		const pack_price = (result.length>0)?result[0].package_price:0;    
		const pack_for = (result.length>0)?result[0].package_for:'';    
		
		const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
		const emailCheckQuery = `SELECT * FROM membership WHERE email1 = '${studAnnual_email}'`;
		const existingUser = await queryDatabase(emailCheckQuery, emailStream);	    	  

      if (existingUser.length > 0) {          		
		return res.status(500).json({ error: 'Email already exists','error_field':'email_id' });	  	             	
      }
      else{  		
				
		const stream = await createSshTunnel(sshTunnelConfig, dbServer);    
        const sqlQuery = `INSERT INTO membership (parent_id,title1,fname1,sname1,dob1,passport1,medical_aid,medical_aid_no,startdate,address1,address2,city1,province1,postcode1,country1,telephone1,telephone2,cellphone1,fax,email1,member_no,verifyEmail,verifyAmount,package_name,package_price,package_for) 
		VALUES ('${instructor_id}','${studAnnual_title}','${studAnnual_fname}','${studAnnual_sname}','${studAnnual_dob}','${studAnnual_passport}','${studAnnual_aid}','${studAnnual_aid_no}','${studAnnual_course}','${studAnnual_address1}','${studAnnual_address2}','${studAnnual_city}','${studAnnual_province}','${studAnnual_post}','${studAnnual_country}','${studAnnual_home}','','${studAnnual_cellphone}','','${studAnnual_email}','${memberNO}','verified','paid','${pack_name}','${pack_price}','${pack_for}')`;  
                
		const result = await queryDatabase(sqlQuery,stream);   		 		  
		const insert_id = result.insertId;  		            	    

		if(insert_id >0)	
		{			 
			// Send a success response
			res.status(200).json({ message: 'Student member data added successfully' });       	     				 		  	
		} 
		else
		{
			res.status(500).json({ error: 'Record could not inserted!' });  	            
		}  
        					  		
      }  
	  
    }
	catch (error) {
		// Handle database or other errors 	  		
		console.error('Error adding data to the database:', error);
		res.status(500).json({ error: error });  
	}	  
	 
  } 
}); 

// payment if membership is expired


// fetch all members
app.get('/fetch_all', async (req, res) => {
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);		
    const sqlQuery = 'SELECT * FROM membership';	   	
    const results = await queryDatabase(sqlQuery, stream);				
    res.status(200).json(results);
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });		    	
  }
}); 

// edit membership
app.get('/edit/:id', async (req, res) => { 
  const id = req.params.id;		 
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);		
    // const sqlQuery = `SELECT * FROM membership WHERE id=${id}`;	     	    
    const sqlQuery = `
      SELECT 
        *,
        (SELECT COUNT(*) FROM membership WHERE parent_id = ${id}) AS parentCount,
        SUM(CASE WHEN membertype = 'diver' THEN 1 ELSE 0 END) AS diverCount,
        SUM(CASE WHEN membertype = 'non_diver' THEN 1 ELSE 0 END) AS nonDiverCount
      FROM 
        membership 
      WHERE 
        id = ${id} OR parent_id = ${id}
    `;
    // console.log(sqlQuery);
    const results = await queryDatabase(sqlQuery, stream);		  		
    res.status(200).json(results);   
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });		    	
  }
}) 

// delete members
app.get('/delete/:id', async (req, res) => {   
  const id = req.params.id;		
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);		
    const sqlQuery = `DELETE FROM membership WHERE id=${id}`;	     	    
    const results = await queryDatabase(sqlQuery, stream);		  		  		
    res.status(200).json(results);   			
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });		    	
  }
})

app.get('/courses', async (req, res) => {
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);		
    const sqlQuery = "SELECT * FROM courses";	          
    const results = await queryDatabase(sqlQuery, stream);	  			
    res.status(200).json(results);
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });		    	
  }
});   



// register save data to membership table -start queries
app.post('/annual_register', async (req, res) => {            
  const data = req.body;                                           

  try {
    // Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    const randomNo = Math.floor(Math.random() * 1000); // Generate a random number
    
    // Insert data into the database  
    const title=(typeof data.title=="undefined")?'':data.title;  	
    const first_name=(typeof data.first_name=="undefined")?'':data.first_name;    
    const last_name=(typeof data.last_name=="undefined")?'':data.last_name;  	
    const dob=(typeof data.dob=="undefined" || data.dob=='' || data.dob== null)?'0000-00-00':data.dob;  
    const passport_no=(typeof data.passport=="undefined")?'':data.passport;   
    const street=(typeof data.street=="undefined")?'':data.street;   
    const city=(typeof data.city=="undefined")?'':data.city; 
    const state=(typeof data.state=="undefined")?'':data.state;  	
    const postal_code=(typeof data.postal_code=="undefined")?'':data.postal_code;    
    const tel_home=(typeof data.tel_home=="undefined")?'':data.tel_home;   
    const tel_work=(typeof data.tel_work=="undefined")?'':data.tel_work;   
    const cell_phone=(typeof data.cell_phone=="undefined")?'':data.cell_phone;    
    const email_address=(typeof data.email_address=="undefined")?'':data.email_address;  	  
    const country=(typeof data.country=="undefined")?'':data.country;   
    const qualification=(typeof data.qualification=="undefined")?'':data.qualification;  	   	
    const agency=(typeof data.agency=="undefined")?'':data.agency;  	 
    const working_as=(typeof data.working_as=="undefined")?'':data.working_as;  	 	
    const diver_type=(typeof data.diver_type=="undefined")?'':data.diver_type;   
    const ref_dive_centre=(typeof data.ref_dive_centre=="undefined")?'':data.ref_dive_centre;  
    const ref_dive_instructor=(typeof data.ref_dive_instructor=="undefined")?'':data.ref_dive_instructor;  
    const aid=(typeof data.aid=="undefined")?'':data.aid;  
    const aid_no=(typeof data.aid_no=="undefined")?'':data.aid_no;    	
    const medication=(typeof data.medication=="undefined")?'':data.medication;    
    const chronic_ill=(typeof data.chronic_ill=="undefined")?'':data.chronic_ill; 
    const start_date=(typeof data.start_date=="undefined" || data.start_date=='' || data.start_date== null)?'0000:00:00':data.start_date;   
    const package_name=(typeof data.package_name=="undefined")?'':data.package_name;  	 	
    const members_add=(typeof data.members_add=="undefined")?'':data.members_add;  
    const payment_opt=(typeof data.payment_opt=="undefined")?'':data.payment_opt;      
    const new_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6);           	
    const kin_title=(typeof data.kin_title=="undefined")?'':data.kin_title;  	
    const kin_fname=(typeof data.kin_fname=="undefined")?'':data.kin_fname;  	  
    const kin_sname=(typeof data.kin_sname=="undefined")?'':data.kin_sname;  
    const kin_dob=(typeof data.kin_dob=="undefined" || data.kin_dob=='' || data.kin_dob== null)?'0000-00-00':data.kin_dob;  
    const kin_passport=(typeof data.kin_passport=="undefined")?'':data.kin_passport;  
    const kin_street=(typeof data.kin_street=="undefined")?'':data.kin_street;  
    const kin_city=(typeof data.kin_city=="undefined")?'':data.kin_city;  
    const kin_state=(typeof data.kin_state=="undefined")?'':data.kin_state;  
    const kin_postal_code=(typeof data.kin_postal_code=="undefined")?'':data.kin_postal_code;  	
    const kin_home=(typeof data.kin_home=="undefined")?'':data.kin_home;  
    const kin_work=(typeof data.kin_work=="undefined")?'':data.kin_work;    
    const kin_mobile=(typeof data.kin_mobile=="undefined")?'':data.kin_mobile;  
    const kin_email_address=(typeof data.kin_email_address=="undefined")?'':data.kin_email_address;    
    const kin_relation=(typeof data.kin_relation=="undefined")?'':data.kin_relation;        	
    const choose_membership='annual';   	   

    let memberPrefix = ''; // Initialize member prefix
    let price = '';
    // Check package_name to determine the member prefix
    if(choose_membership === 'annual'){
      if (package_name === 'Standard') {
          memberPrefix = 'DANS';
          price = 1940;
      } else if (package_name === 'Plus') {
          memberPrefix = 'DANP';
          price = 2395;
      } else if (package_name === 'Master Dive Pro') {
          memberPrefix = 'DANM';
          price = 2565;
      }else if (package_name === 'Master Tech') {
        memberPrefix = 'DANX';
        price = 2735;
      }else if (package_name === 'Master Freediver Pro') {
        memberPrefix = 'DANF';
        price = 2565;
      }
    }else if(choose_membership === 'student'){
      memberPrefix = `DANT`;
    }
    else if(choose_membership === 'temporary'){
      memberPrefix = `DANE`;
    }
    else if(choose_membership === 'commercial'){
      if (package_name === 'individual') {
        memberPrefix = 'DANCI';
      } else if (package_name === 'professional') {
          memberPrefix = 'DANCP';
      } else if (package_name === 'company') {
          memberPrefix = 'DANCC';
      }else if (package_name === 'student') {
        memberPrefix = 'DANCS';
      }
    }
    
    const memberNO = `${memberPrefix} ${randomNo}`; // Generate random number and the determined prefix
    const package_price = `${price}`;

    // Check if the email already exists in the database
    const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
    const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${email_address}'`;
    const existingUser = await queryDatabase(emailCheckQuery, emailStream);

    if (existingUser.length > 0) {        
	  return res.status(500).json({ error: 'Email already exist','error_field':'email_address' });	  
    }
    else{  	    

	  const sqlQuery = `INSERT INTO membership (title1,fname1,sname1,dob1,passport1,address1,city1,province1,postcode1,telephone1,telephone2,cellphone1,email1,country1,qualification,agency,working_as,diver_type,ref_dive_centre,ref_dive_instructor,medical_aid,medical_aid_no,medication,chronic_ill,startdate,package_name, membership_slot,pay_method,password,choose_membership,member_no,title2,fname2,sname2,dob2,passport2,address3,city2,province2,postcode2,telephone3,telephone4,cellphone2,email2,relation,package_for,package_price,verifyEmail,verifyAmount,status1) 
	  VALUES ('${title}','${first_name}','${last_name}','${dob}','${passport_no}','${street}','${city}','${state}','${postal_code}','${tel_home}','${tel_work}','${cell_phone}','${email_address}','${country}','${qualification}','${agency}','${working_as}','${diver_type}','${ref_dive_centre}','${ref_dive_instructor}', '${aid}','${aid_no}','${medication}','${chronic_ill}','${start_date}','${package_name}','${members_add}','${payment_opt}','${new_pwd}','${choose_membership}','${memberNO}','${kin_title}','${kin_fname}','${kin_sname}','${kin_dob}','${kin_passport}','${kin_street}','${kin_city}','${kin_state}','${kin_postal_code}','${kin_home}','${kin_work}','${kin_mobile}','${kin_email_address}','${kin_relation}','year','${package_price}','verified','not_paid','INACTIVE')`;      	
	  
	  const result = await queryDatabase(sqlQuery,stream);   				  		 		
	  const insert_id = result.insertId; 
	  
	   if(insert_id >0)			   
		{
			// Send a success response	 			
			for(var i=1;i<=4;i++)
			{ 				
				const family_title=(typeof data[`family_title${i}`]=="undefined")?'':data[`family_title${i}`];  	
				const family_package=(typeof data[`family_packageName${i}`]=="undefined")?'':data[`family_packageName${i}`];  				
				const family_fname=(typeof data[`family_fname${i}`]=="undefined")?'':data[`family_fname${i}`];  
				const family_sname=(typeof data[`family_sname${i}`]=="undefined")?'':data[`family_sname${i}`];  
				const family_dob=(typeof data[`family_dob${i}`]=="undefined" || data[`family_dob${i}`]=='' || data[`family_dob${i}`]== null)?'0000-00-00':data[`family_dob${i}`];  
				const family_passport=(typeof data[`family_passport${i}`]=="undefined")?'':data[`family_passport${i}`];  
				const family_email=(typeof data[`family_email${i}`]=="undefined")?'':data[`family_email${i}`];  
				const family_phone=(typeof data[`family_phone${i}`]=="undefined")?'':data[`family_phone${i}`];  				
				const family_diver=(typeof data[`family_diver${i}`]=="undefined")?'':data[`family_diver${i}`];  
				const family_type = (family_diver=='Yes')?'diver':'non_diver';											

				const diver_type=(typeof data[`diver_type${i}`]=="undefined")?'':data[`diver_type${i}`];  
				const family_workingAs=(typeof data[`family_workingAs${i}`]=="undefined")?'':data[`family_workingAs${i}`];  	
				
				const family_medication=(typeof data[`family_medication${i}`]=="undefined")?'':data[`family_medication${i}`]; 
				const family_chronic=(typeof data[`family_chronic${i}`]=="undefined")?'':data[`family_chronic${i}`];   
        const family_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6);           	

        const randomNo2 = Math.floor(Math.random() * 1000); // Generate a random number
        let memberPrefix2 = ''; // Initialize member prefix
        let price2 = '';

        if (family_package === 'Standard') {
          memberPrefix2 = 'DANS';
          price2 = 1940;
        } else if (family_package === 'Plus') {
            memberPrefix2 = 'DANP';
            price2 = 2395;
        } else if (family_package === 'Master Dive Pro') {
            memberPrefix2 = 'DANM';
            price2 = 2565;
        }else if (family_package === 'Master Tech') {
          memberPrefix2 = 'DANX';
          price2 = 2735;
        }else if (family_package === 'Master Freediver Pro') {
          memberPrefix2 = 'DANF';
          price2 = 2565;
        }

        const memberNO2 = `${memberPrefix2} ${randomNo2}`; // Generate random number
        const family_price = `${price2}`;
				
				const stream = await createSshTunnel(sshTunnelConfig,dbServer);  

				if(family_title !='' || family_package !='' || family_fname !='' || family_sname !='' || family_dob !='0000-00-00' || family_passport !='' || family_email !='' || family_phone !='' || family_diver !='' || diver_type !='' || family_workingAs !='' || family_medication !='' || family_chronic !='')
				{
					const sqlQuery = `INSERT INTO membership (parent_id,title1,package_name,membertype,fname1,sname1,dob1,passport1,email1,cellphone1,permission,diver_type,working_as,medication,chronic_ill,choose_membership,package_for,package_price,verifyEmail,verifyAmount,status1,member_no,password) 
					VALUES ('${insert_id}','${family_title}','${family_package}','${family_type}','${family_fname}','${family_sname}','${family_dob}','${family_passport}','${family_email}','${family_phone}','${family_diver}','${diver_type}','${family_workingAs}','${family_medication}','${family_chronic}','annual','year','${family_price}','verified','paid','ACTIVE','${memberNO2}','${family_pwd}')`;               
				
					await queryDatabase(sqlQuery,stream); 			  
				}
				  				
			} 	 	
		  sendForgotEmail(email_address, new_pwd);	 
			res.status(200).json({ message: 'Members registered successfully. Check your email for verification.' });  
		} 
		else
		{
			res.status(500).json({ error: 'Record could not inserted!' });  	            
		} 	 
		
      
    }
	 
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error });
  } 
});
app.post('/student_register', async (req, res) => {            
  const data = req.body;                                           

  try {
    // Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    const randomNo = Math.floor(Math.random() * 1000); // Generate a random number
    
    // Insert data into the database   	
    const first_name=(typeof data.first_name=="undefined")?'':data.first_name;    
    const last_name=(typeof data.last_name=="undefined")?'':data.last_name;  	
    const dob=(typeof data.dob=="undefined" || data.dob=='' || data.dob== null)?'0000-00-00':data.dob;  
    const passport_no=(typeof data.passport=="undefined")?'':data.passport;   
    const cell_phone=(typeof data.cell_phone=="undefined")?'':data.cell_phone;    
    const email_address=(typeof data.email_address=="undefined")?'':data.email_address;  	  
    const country=(typeof data.country=="undefined")?'':data.country;   
    const street=(typeof data.street=="undefined")?'':data.street;   
    const city=(typeof data.city=="undefined")?'':data.city; 
    const postal_code=(typeof data.postal_code=="undefined")?'':data.postal_code;    
    const instructor_name=(typeof data.instructor_name=="undefined")?'':data.instructor_name;  	
    const instructor_dan_no=(typeof data.instructor_dan_no=="undefined")?'':data.instructor_dan_no;   
    const ref_dive_centre=(typeof data.referral_dive=="undefined")?'':data.referral_dive;  
    const course_start=(typeof data.course_start=="undefined" || data.course_start=='' || data.course_start== null)?'0000-00-00':data.course_start;  
    const package_name=(typeof data.course_type=="undefined")?'':data.course_type;  	 	
    const rstc=(typeof data.rstc=="undefined")?'':data.rstc;  
    const aid=(typeof data.aid=="undefined")?'':data.aid;  
    const aid_no=(typeof data.aid_no=="undefined")?'':data.aid_no;    	
    const medication=(typeof data.medication=="undefined")?'':data.medication;    
    const chronic_ill=(typeof data.chronic_ill=="undefined")?'':data.chronic_ill;    
    const new_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6);           	
    const choose_membership='student';   	   

    const memberNO = `DANT ${randomNo}`; // Generate random number and the determined prefix

    // Check if the email already exists in the database
    const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
    const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${email_address}'`;
    const existingUser = await queryDatabase(emailCheckQuery, emailStream);

    if (existingUser.length > 0) {        
	  return res.status(500).json({ error: 'Email already exist','error_field':'email_address' });	  
    }
    else{  	    

      const sqlQuery = `INSERT INTO membership (fname1,sname1,dob1,passport1,address1,city1,postcode1,cellphone1,email1,country1,contact_person,contact_phone,ref_dive_centre,medical_aid,medical_aid_no,medication,chronic_ill,startdate,package_name,password,choose_membership,member_no,package_for,package_price,verifyEmail,verifyAmount,status1,rstc) 
      VALUES ('${first_name}','${last_name}','${dob}','${passport_no}','${street}','${city}','${postal_code}','${cell_phone}','${email_address}','${country}','${instructor_name}','${instructor_dan_no}','${ref_dive_centre}', '${aid}','${aid_no}','${medication}','${chronic_ill}','${course_start}','${package_name}','${new_pwd}','${choose_membership}','${memberNO}','year','Free','verified','not_paid','INACTIVE','${rstc}')`;      	
      
      await queryDatabase(sqlQuery,stream);   				  		 		

      sendForgotEmail(email_address, new_pwd);	 
      res.status(200).json({ message: 'Members registered successfully. Check your email for verification.' });  
    }
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error });
  } 
});
app.post('/student_instructor_register', async (req, res) => {            
  const data = req.body;                                           

  try {       
    // Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    const randomNo = Math.floor(Math.random() * 1000); // Generate a random number

    const first_name=(typeof data.first_name=="undefined")?'':data.first_name;    
    const last_name=(typeof data.last_name=="undefined")?'':data.last_name;  	
    const cell_phone=(typeof data.cell_phone=="undefined")?'':data.cell_phone;    
    const email_address=(typeof data.email_address=="undefined")?'':data.email_address;  	  
    const dan_no=(typeof data.dan_no=="undefined")?'':data.dan_no;   
    const ref_dive_centre=(typeof data.ref_dive_centre=="undefined")?'':data.ref_dive_centre;  
    const course_date=(typeof data.course_date=="undefined" || data.course_date=='' || data.course_date== null)?'0000:00:00':data.course_date;   
    const rstc=(typeof data.rstc=="undefined")?'':data.rstc;        
    const new_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6);           	
    const package_name='Instructor';  	 	
    const choose_membership='student_instructor';   	   

    const memberNO = `DANT ${randomNo}`; // Generate random number and the determined prefix

    // Check if the email already exists in the database
    const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
    const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${email_address}'`;
    const existingUser = await queryDatabase(emailCheckQuery, emailStream);

    if (existingUser.length > 0) {        
	  return res.status(500).json({ error: 'Email already exist','error_field':'email_address' });	  
    }
    else{  	             

	  const sqlQuery = `INSERT INTO membership (fname1,sname1,cellphone1,email1,ref_dive_centre,startdate,package_name,password,choose_membership,member_no,package_for,package_price,verifyEmail,verifyAmount,status1,rstc,contact_phone) 
	  VALUES ('${first_name}','${last_name}','${cell_phone}','${email_address}','${ref_dive_centre}','${course_date}','${package_name}','${new_pwd}','${choose_membership}','${memberNO}','year','Free','verified','not_paid','INACTIVE','${rstc}','${dan_no}')`;      	
	  
	  const result = await queryDatabase(sqlQuery,stream);   				  		 		
	  const insert_id = result.insertId; 
	  
	   if(insert_id >0)			   
		{
			// Send a success response	 			
			for(var i=1;i<=5;i++)
			{ 				 					
				const stud_fname=(typeof data[`stud_fname${i}`]=="undefined")?'':data[`stud_fname${i}`];  
				const stud_lname=(typeof data[`stud_lname${i}`]=="undefined")?'':data[`stud_lname${i}`];  
				const stud_dob=(typeof data[`stud_dob${i}`]=="undefined" || data[`stud_dob${i}`]=='' || data[`stud_dob${i}`]== null)?'0000-00-00':data[`stud_dob${i}`];  
				const stud_passport=(typeof data[`stud_passport${i}`]=="undefined")?'':data[`stud_passport${i}`];  
				const stud_phone=(typeof data[`stud_phone${i}`]=="undefined")?'':data[`stud_phone${i}`];  				
				const stud_email=(typeof data[`stud_email${i}`]=="undefined")?'':data[`stud_email${i}`];  
				const stud_country=(typeof data[`stud_country${i}`]=="undefined")?'':data[`stud_country${i}`];  
				const stud_street=(typeof data[`stud_street${i}`]=="undefined")?'':data[`stud_street${i}`];  
				const stud_city=(typeof data[`stud_city${i}`]=="undefined")?'':data[`stud_city${i}`];  
				const stud_postal=(typeof data[`stud_postal${i}`]=="undefined")?'':data[`stud_postal${i}`];  
				const stud_course=(typeof data[`stud_course${i}`]=="undefined")?'':data[`stud_course${i}`];  
				const stud_aid=(typeof data[`stud_aid${i}`]=="undefined")?'':data[`stud_aid${i}`];  
				const stud_aid_no=(typeof data[`stud_aid_no${i}`]=="undefined")?'':data[`stud_aid_no${i}`];  
				const stud_medication=(typeof data[`stud_medication${i}`]=="undefined")?'':data[`stud_medication${i}`];  
				const stud_chronic=(typeof data[`stud_chronic${i}`]=="undefined")?'':data[`stud_chronic${i}`];  											
				const stud_permission=(typeof data[`stud_permission${i}`]=="undefined")?'':data[`stud_permission${i}`];  

        const randomNo2 = Math.floor(Math.random() * 1000); // Generate a random number
        const memberNO2 = `DANT ${randomNo2}`; // Generate random number and the determined prefix
				
				const stream = await createSshTunnel(sshTunnelConfig,dbServer);  

				if(stud_fname !='' || stud_lname !='' || stud_dob !='0000-00-00' || stud_passport !='' || stud_phone !='' || stud_email !='' || stud_country !='' || stud_street !='' || stud_city !='' || stud_postal !='' || stud_course !='' || stud_aid !='' || stud_aid_no !='' || stud_medication !='' || stud_chronic !='' || stud_permission !='')    
				{
					const sqlQuery = `INSERT INTO membership (parent_id,fname1,sname1,dob1,passport1,cellphone1,email1,country1,address1,city1,postcode1,package_name,package_price,package_for,medical_aid,medical_aid_no,medication,chronic_ill,permission,choose_membership,verifyEmail,verifyAmount,status1,member_no) 
					VALUES ('${insert_id}','${stud_fname}','${stud_lname}','${stud_dob}','${stud_passport}','${stud_phone}','${stud_email}','${stud_country}','${stud_street}','${stud_city}','${stud_postal}','${stud_course}','Free','year','${stud_aid}','${stud_aid_no}','${stud_medication}','${stud_chronic}','${stud_permission}','student','verified','paid','ACTIVE','${memberNO2}')`;                 
				
					await queryDatabase(sqlQuery,stream); 			  
				}  			
			} 	 	
		  sendForgotEmail(email_address, new_pwd);	 
			res.status(200).json({ message: 'Members registered successfully. Check your email for verification.' });    
		} 
		else
		{
			res.status(500).json({ error: 'Record could not inserted!' });  	            
		} 	 
		            
    }
	 
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error });
  } 
});
app.post('/temporary_register', async (req, res) => {            
  const data = req.body;                                                   

  try {
    // Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    const randomNo = Math.floor(Math.random() * 1000); // Generate a random number
    
    // Insert data into the database  
    const title=(typeof data.title=="undefined")?'':data.title;  	
    const first_name=(typeof data.first_name=="undefined")?'':data.first_name;    
    const last_name=(typeof data.last_name=="undefined")?'':data.last_name;  	
    const dob=(typeof data.dob=="undefined" || data.dob=='' || data.dob== null)?'0000-00-00':data.dob;  
    const passport_no=(typeof data.passport=="undefined")?'':data.passport;   
    const street=(typeof data.street=="undefined")?'':data.street;   
    const city=(typeof data.city=="undefined")?'':data.city; 
    const state=(typeof data.state=="undefined")?'':data.state;  	                 
    const postal_code=(typeof data.postal_code=="undefined")?'':data.postal_code;    
    const tel_home=(typeof data.tel_home=="undefined")?'':data.tel_home;   
    const tel_work=(typeof data.tel_work=="undefined")?'':data.tel_work;   
    const cell_phone=(typeof data.cell_phone=="undefined")?'':data.cell_phone;    
    const email_address=(typeof data.email_address=="undefined")?'':data.email_address;  	  
    const country=(typeof data.country=="undefined")?'':data.country;  
    const dive_country=(typeof data.dive_country=="undefined")?'':data.dive_country;   
    const dive_start=(typeof data.dive_start=="undefined" || data.dive_start=='' || data.dive_start== null)?'0000-00-00':data.dive_start;  
    const dive_end=(typeof data.dive_end=="undefined" || data.dive_end=='' || data.dive_end== null)?'0000-00-00':data.dive_end;  
    const qualification=(typeof data.qualification=="undefined")?'':data.qualification;  	   	
    const agency=(typeof data.agency=="undefined")?'':data.agency;  	 
    const working_as=(typeof data.working_as=="undefined")?'':data.working_as;  	 	
    const diver_type=(typeof data.diver_type=="undefined")?'':data.diver_type;   
    const ref_dive_centre=(typeof data.ref_dive_centre=="undefined")?'':data.ref_dive_centre;  
    const ref_dive_instructor=(typeof data.ref_dive_instructor=="undefined")?'':data.ref_dive_instructor;  
    const aid=(typeof data.aid=="undefined")?'':data.aid;       
    const aid_no=(typeof data.aid_no=="undefined")?'':data.aid_no;    	
    const medication=(typeof data.medication=="undefined")?'':data.medication;                 
    const chronic_ill=(typeof data.chronic_ill=="undefined")?'':data.chronic_ill;                            
    const kin_title=(typeof data.kin_title=="undefined")?'':data.kin_title;  	
    const kin_fname=(typeof data.kin_fname=="undefined")?'':data.kin_fname;  	  
    const kin_sname=(typeof data.kin_sname=="undefined")?'':data.kin_sname;  
    const kin_dob=(typeof data.kin_dob=="undefined" || data.kin_dob=='' || data.kin_dob== null)?'0000-00-00':data.kin_dob;  
    const kin_passport=(typeof data.kin_passport=="undefined")?'':data.kin_passport;  
    const kin_street=(typeof data.kin_street=="undefined")?'':data.kin_street;  
    const kin_city=(typeof data.kin_city=="undefined")?'':data.kin_city;  
    const kin_state=(typeof data.kin_state=="undefined")?'':data.kin_state;  
    const kin_postal_code=(typeof data.kin_postal_code=="undefined")?'':data.kin_postal_code;  	
    const kin_home=(typeof data.kin_home=="undefined")?'':data.kin_home;  
    const kin_work=(typeof data.kin_work=="undefined")?'':data.kin_work;    
    const kin_mobile=(typeof data.kin_mobile=="undefined")?'':data.kin_mobile;       
    const kin_email_address=(typeof data.kin_email_address=="undefined")?'':data.kin_email_address;    
    const kin_relation=(typeof data.kin_relation=="undefined")?'':data.kin_relation;        	
    const choose_membership='temporary'; 
    const package_name=(typeof data.package_name=="undefined")?'':data.package_name;   
    const payment_opt=(typeof data.payment_opt=="undefined")?'':data.payment_opt;      
    const new_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6);                                          	   

    let memberPrefix = ''; // Initialize member prefix
    let price = '';
    let package_for = '';
    // Check package_name to determine the member prefix
   
    if(choose_membership === 'temporary'){
      memberPrefix = `DANE`;
    }

    if(package_name === '1 Day Package'){
      price = 85;
      package_for = '1 day';
    }else if(package_name === '2 Day Package'){
      price = 170;
      package_for = '2 day';
    }else if(package_name === '3 Day Package'){
      price = 255;
      package_for = '3 day';
    }else if(package_name === '4 Day Package'){
      price = 340;
      package_for = '4 day';
    }else if(package_name === '5 Day Package'){
      price = 425;
      package_for = '5 day';
    }else if(package_name === '10 Day Package'){
      price = 850;
      package_for = '10 day';
    }else if(package_name === '15 Day Package'){
      price = 1275;
      package_for = '15 day';
    }
    
    const memberNO = `${memberPrefix} ${randomNo}`; // Generate random number and the determined prefix
    const package_price = `${price}`;
    const packageFor = `${package_for}`;

    // Check if the email already exists in the database
    const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
    const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${email_address}'`;
    const existingUser = await queryDatabase(emailCheckQuery, emailStream);

    if (existingUser.length > 0) {        
	  return res.status(500).json({ error: 'Email already exist','error_field':'email_address' });	  
    }
    else{  	    

	  const sqlQuery = `INSERT INTO membership (title1,fname1,sname1,dob1,passport1,address1,city1,province1,postcode1,telephone1,telephone2,cellphone1,email1,country1,qualification,agency,working_as,diver_type,ref_dive_centre,ref_dive_instructor,medical_aid,medical_aid_no,medication,chronic_ill,package_name,pay_method,password,choose_membership,member_no,title2,fname2,sname2,dob2,passport2,address2,city2,province2,postcode2,telephone3,telephone4,cellphone2,email2,relation,package_for,package_price,verifyEmail,verifyAmount,status1,dive_location,startdate,lastdate) 
	  VALUES ('${title}','${first_name}','${last_name}','${dob}','${passport_no}','${street}','${city}','${state}','${postal_code}','${tel_home}','${tel_work}','${cell_phone}','${email_address}','${country}','${qualification}','${agency}','${working_as}','${diver_type}','${ref_dive_centre}','${ref_dive_instructor}', '${aid}','${aid_no}','${medication}','${chronic_ill}','${package_name}','${payment_opt}','${new_pwd}','${choose_membership}','${memberNO}','${kin_title}','${kin_fname}','${kin_sname}','${kin_dob}','${kin_passport}','${kin_street}','${kin_city}','${kin_state}','${kin_postal_code}','${kin_home}','${kin_work}','${kin_mobile}','${kin_email_address}','${kin_relation}','${packageFor}','${package_price}','verified','not_paid','INACTIVE','${dive_country}','${dive_start}','${dive_end}')`;      	
	  
	  const result = await queryDatabase(sqlQuery,stream);   				  		 		

			// Send a success response	 			
		  sendForgotEmail(email_address, new_pwd);	 
			res.status(200).json({ message: 'Members registered successfully. Check your email for verification.' });   
    }
	 
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error });
  } 
});
app.post('/temporary_group_register', async (req, res) => {            
  const data = req.body;                                           

  try {
    // Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    const randomNo = Math.floor(Math.random() * 1000); // Generate a random number
    
    // Insert data into the database   
    const ref_dive_centre=(typeof data.ref_dive_centre=="undefined")?'':data.ref_dive_centre;  
    const ref_dive_instructor=(typeof data.ref_dive_instructor=="undefined")?'':data.ref_dive_instructor;  
    const leader_name=(typeof data.leader_name=="undefined")?'':data.leader_name;  
    const leader_phone=(typeof data.leader_phone=="undefined")?'':data.leader_phone;    	
    const dive_country=(typeof data.dive_country=="undefined")?'':data.dive_country;    
    const dive_start=(typeof data.dive_start=="undefined" || data.dive_start=='' || data.dive_start== null)?'0000-00-00':data.dive_start;  
    const dive_end=(typeof data.dive_end=="undefined" || data.dive_end=='' || data.dive_end== null)?'0000-00-00':data.dive_end;  
    const package_name=(typeof data.package_name=="undefined")?'':data.package_name;  	 	 
    const payment_opt=(typeof data.payment_opt=="undefined")?'':data.payment_opt;      
    const new_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6);           	       	
    const choose_membership='temporary_group';   	   
    
    let memberPrefix = ''; // Initialize member prefix
    let price = '';
    let package_for = '';
    // Check package_name to determine the member prefix
   
    if(choose_membership === 'temporary_group'){
      memberPrefix = `DANE`;
    }

    if(package_name === '1 Day Package'){
      price = 85;
      package_for = '1 day';
    }else if(package_name === '2 Day Package'){
      price = 170;
      package_for = '2 day';
    }else if(package_name === '3 Day Package'){
      price = 255;
      package_for = '3 day';
    }else if(package_name === '4 Day Package'){
      price = 340;
      package_for = '4 day';
    }else if(package_name === '5 Day Package'){
      price = 425;
      package_for = '5 day';
    }else if(package_name === '10 Day Package'){
      price = 850;
      package_for = '10 day';
    }else if(package_name === '15 Day Package'){
      price = 1275;
      package_for = '15 day';
    }
    
    const memberNO = `${memberPrefix} ${randomNo}`; // Generate random number and the determined prefix
    const package_price = `${price}`;
    const packageFor = `${package_for}`;

	  const sqlQuery = `INSERT INTO membership (ref_dive_centre,ref_dive_instructor,contact_person,contact_phone,dive_location,startdate,lastdate, package_name,pay_method,password,choose_membership,member_no, package_for,package_price,verifyEmail,verifyAmount,status1) 
	  VALUES ('${ref_dive_centre}','${ref_dive_instructor}', '${leader_name}','${leader_phone}','${dive_country}','${dive_start}','${dive_end}','${package_name}','${payment_opt}','${new_pwd}','${choose_membership}','${memberNO}', '${packageFor}','${package_price}','verified','not_paid','INACTIVE')`;      	
	  
	  const result = await queryDatabase(sqlQuery,stream);   				  		 		
	  const insert_id = result.insertId; 
	  
      if(insert_id >0)			   
      {
        // Send a success response	 			
        for(var i=1;i<=5;i++)
        { 				
          const diver_title=(typeof data[`diver_title${i}`]=="undefined")?'':data[`diver_title${i}`];  	
          const diver_fname=(typeof data[`diver_fname${i}`]=="undefined")?'':data[`diver_fname${i}`];  
          const diver_sname=(typeof data[`diver_sname${i}`]=="undefined")?'':data[`diver_sname${i}`];  
          const diver_dob=(typeof data[`diver_dob${i}`]=="undefined" || data[`diver_dob${i}`]=='' || data[`diver_dob${i}`]== null)?'0000-00-00':data[`diver_dob${i}`];  
          const diver_passport=(typeof data[`diver_passport${i}`]=="undefined")?'':data[`diver_passport${i}`];  
          const diver_phone=(typeof data[`diver_phone${i}`]=="undefined")?'':data[`diver_phone${i}`];  				
          const diver_email=(typeof data[`diver_email${i}`]=="undefined")?'':data[`diver_email${i}`];  
          const diver_country=(typeof data[`diver_country${i}`]=="undefined")?'':data[`diver_country${i}`];  
          const diver_qualification=(typeof data[`diver_qualification${i}`]=="undefined")?'':data[`diver_qualification${i}`];  
          const diver_agency=(typeof data[`diver_agency${i}`]=="undefined")?'':data[`diver_agency${i}`];  
          const diver_type=(typeof data[`diver_type${i}`]=="undefined")?'':data[`diver_type${i}`];  
          const diver_aid=(typeof data[`diver_aid${i}`]=="undefined")?'':data[`diver_aid${i}`];  
          const diver_aid_no=(typeof data[`diver_aid_no${i}`]=="undefined")?'':data[`diver_aid_no${i}`];  
          const diver_medication=(typeof data[`diver_medication${i}`]=="undefined")?'':data[`diver_medication${i}`]; 
          const diver_chronic=(typeof data[`diver_chronic${i}`]=="undefined")?'':data[`diver_chronic${i}`];   
          const diver_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6);   
    
          const stream = await createSshTunnel(sshTunnelConfig,dbServer);  

          if(diver_title !='' || diver_fname !='' || diver_sname !='' || diver_dob !='0000-00-00' || diver_passport !='' || diver_email !='' || diver_phone !='' || diver_type !='' || diver_medication !='' || diver_chronic !='' || diver_country !='' || diver_qualification !='' || diver_agency !='' || diver_chronic !='' || diver_chronic !='' || diver_chronic !='' )        
          {
            const sqlQuery = `INSERT INTO membership (parent_id,title1,fname1,sname1,dob1,passport1,email1,cellphone1,country1,diver_type,medication,chronic_ill,choose_membership,verifyEmail,verifyAmount,status1,member_no,password,qualification,agency,medical_aid,medical_aid_no) 
            VALUES ('${insert_id}','${diver_title}','${diver_fname}','${diver_sname}','${diver_dob}','${diver_passport}','${diver_email}','${diver_phone}','${diver_country}','${diver_type}','${diver_medication}','${diver_chronic}','${choose_membership}','verified','paid','ACTIVE','${memberNO}','${diver_pwd}','${diver_qualification}','${diver_agency}','${diver_aid}','${diver_aid_no}')`;               
          
            await queryDatabase(sqlQuery,stream); 			  
          }
                    
        } 	 	
        // sendForgotEmail(email_address, new_pwd);	 
        res.status(200).json({ message: 'Members registered successfully. Check your email for verification.' });  
      } 
      else
      {
        res.status(500).json({ error: 'Record could not inserted!' });  	            
      } 	 
    
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error, error:error.message });
  } 
});
app.post('/commercial_diver_register', async (req, res) => {            
  const data = req.body;                                           

  try {
    // Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    const randomNo = Math.floor(Math.random() * 1000); // Generate a random number
    
    // Insert data into the database  
    const title=(typeof data.title=="undefined")?'':data.title;  	
    const first_name=(typeof data.first_name=="undefined")?'':data.first_name;    
    const last_name=(typeof data.last_name=="undefined")?'':data.last_name;  	
    const dob=(typeof data.dob=="undefined" || data.dob=='' || data.dob== null)?'0000-00-00':data.dob;  
    const passport_no=(typeof data.passport=="undefined")?'':data.passport;   
    const street=(typeof data.street=="undefined")?'':data.street;   
    const city=(typeof data.city=="undefined")?'':data.city; 
    const state=(typeof data.state=="undefined")?'':data.state;  	
    const postal_code=(typeof data.postal_code=="undefined")?'':data.postal_code;    
    const tel_home=(typeof data.tel_home=="undefined")?'':data.tel_home;   
    const tel_work=(typeof data.tel_work=="undefined")?'':data.tel_work;   
    const cell_phone=(typeof data.cell_phone=="undefined")?'':data.cell_phone;    
    const email_address=(typeof data.email_address=="undefined")?'':data.email_address;  	  
    const country=(typeof data.country=="undefined")?'':data.country;   
    const glass_type=(typeof data.glass_type=="undefined")?'':data.glass_type;  
    const reg_no=(typeof data.reg_no=="undefined")?'':data.reg_no;  	   	
    const diver_type=(typeof data.diver_type=="undefined")?'':data.diver_type;   
    const doctor_name=(typeof data.doctor_name=="undefined")?'':data.doctor_name;   
    const doctor_contact=(typeof data.doctor_contact=="undefined")?'':data.doctor_contact; 

    let medicalFit = '';
    const medical_fit = (typeof data.medical_fit === "undefined") ? '' : data.medical_fit;  	 
    if (medical_fit === 'Other') {
      medicalFit = (typeof data.medicalFitInputValue === "undefined") ? '' : data.medicalFitInputValue;
    } else {
      medicalFit = medical_fit;
    }

    let medicalRestriction = '';
    const medical_restriction = (typeof data.medical_restriction === "undefined") ? '' : data.medical_restriction;  	 
    if (medical_restriction === 'Other') {
      medicalRestriction = (typeof data.medicalRestrictionInputValue === "undefined") ? '' : data.medicalRestrictionInputValue;
    } else {
      medicalRestriction = medical_restriction;
    }

    let caseDcs = '';
    const case_dcs = (typeof data.case_dcs === "undefined") ? '' : data.case_dcs;  	 
    if (case_dcs === 'Other') {
      caseDcs = (typeof data.caseDcsInputValue === "undefined") ? '' : data.caseDcsInputValue;
    } else {
      caseDcs = case_dcs;
    }
    const medication=(typeof data.medication=="undefined")?'':data.medication;    
    const chronic_ill=(typeof data.chronic_ill=="undefined")?'':data.chronic_ill;  
    const aid=(typeof data.aid=="undefined")?'':data.aid;  
    const aid_no=(typeof data.aid_no=="undefined")?'':data.aid_no; 
    const emer_name=(typeof data.emer_name=="undefined")?'':data.emer_name;    	
    const emer_phone=(typeof data.emer_phone=="undefined")?'':data.emer_phone;    	
    const kin_title=(typeof data.kin_title=="undefined")?'':data.kin_title;  	
    const kin_fname=(typeof data.kin_fname=="undefined")?'':data.kin_fname;  	  
    const kin_sname=(typeof data.kin_sname=="undefined")?'':data.kin_sname;  
    const kin_dob=(typeof data.kin_dob=="undefined" || data.kin_dob=='' || data.kin_dob== null)?'0000-00-00':data.kin_dob;  
    const kin_passport=(typeof data.kin_passport=="undefined")?'':data.kin_passport;  
    const kin_street=(typeof data.kin_street=="undefined")?'':data.kin_street;  
    const kin_city=(typeof data.kin_city=="undefined")?'':data.kin_city;  
    const kin_state=(typeof data.kin_state=="undefined")?'':data.kin_state;  
    const kin_postal_code=(typeof data.kin_postal_code=="undefined")?'':data.kin_postal_code;  	
    const kin_home=(typeof data.kin_home=="undefined")?'':data.kin_home;  
    const kin_work=(typeof data.kin_work=="undefined")?'':data.kin_work;    
    const kin_mobile=(typeof data.kin_mobile=="undefined")?'':data.kin_mobile;  
    const kin_email_address=(typeof data.kin_email_address=="undefined")?'':data.kin_email_address; 
    const kin_relation=(typeof data.kin_relation=="undefined")?'':data.kin_relation;        	
    const start_date=(typeof data.start_date=="undefined" || data.start_date=='' || data.start_date== null)?'0000:00:00':data.start_date;    	 	
    const payment_opt=(typeof data.payment_opt=="undefined")?'':data.payment_opt;      
    const new_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6);           	
    const choose_membership='commercial_diver';   
    const package_name='individual';   	   


    let memberPrefix = ''; // Initialize member prefix

      if (package_name === 'individual') {
        memberPrefix = 'DANCI';
      } else if (package_name === 'professional') {
          memberPrefix = 'DANCP';
      } else if (package_name === 'company') {
          memberPrefix = 'DANCC';
      }else if (package_name === 'student') {
        memberPrefix = 'DANCS';
      }
    
    const memberNO = `${memberPrefix} ${randomNo}`; // Generate random number and the determined prefix

    // Check if the email already exists in the database
    const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
    const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${email_address}'`;
    const existingUser = await queryDatabase(emailCheckQuery, emailStream);

    if (existingUser.length > 0) {        
	  return res.status(500).json({ error: 'Email already exist','error_field':'email_address' });	  
    }
    else{  	    

	  const sqlQuery = `INSERT INTO membership (title1,fname1,sname1,dob1,passport1,address1,city1,province1,postcode1,telephone1,telephone2,cellphone1,email1,country1,glass_type,registration_no,diver_type,doctor_name,doctor_phone,medical_fit,medical_restriction,case_dcs,medical_aid,medical_aid_no,medication,chronic_ill,contact_person,contact_phone,title2,fname2,sname2,dob2,passport2,address3,city2,province2,postcode2,telephone3,telephone4,cellphone2,email2,relation,startdate,pay_method,choose_membership,package_name,password,member_no,package_for,verifyEmail,verifyAmount,status1) 
	  VALUES ('${title}','${first_name}','${last_name}','${dob}','${passport_no}','${street}','${city}','${state}','${postal_code}','${tel_home}','${tel_work}','${cell_phone}','${email_address}','${country}','${glass_type}','${reg_no}','${diver_type}','${doctor_name}','${doctor_contact}','${medicalFit}','${medicalRestriction}','${caseDcs}','${aid}','${aid_no}','${medication}','${chronic_ill}','${emer_name}','${emer_phone}','${kin_title}','${kin_fname}','${kin_sname}','${kin_dob}','${kin_passport}','${kin_street}','${kin_city}','${kin_state}','${kin_postal_code}','${kin_home}','${kin_work}','${kin_mobile}','${kin_email_address}','${kin_relation}','${start_date}','${payment_opt}','${choose_membership}','${package_name}','${new_pwd}','${memberNO}','year','verified','not_paid','INACTIVE')`;      	
	  
	  await queryDatabase(sqlQuery,stream);   				  		 		

		sendForgotEmail(email_address, new_pwd);	 
		res.status(200).json({ message: 'Members registered successfully. Check your email for verification.' });  
    }
	 
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error, error:error.message });
  } 
});
app.post('/professional_register', async (req, res) => {            
  const data = req.body;                                                   

  try {
    // Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    const randomNo = Math.floor(Math.random() * 1000); // Generate a random number
    
    // Insert data into the database  
    const title=(typeof data.title=="undefined")?'':data.title;  	
    const first_name=(typeof data.first_name=="undefined")?'':data.first_name;    
    const last_name=(typeof data.last_name=="undefined")?'':data.last_name;  	
    const dob=(typeof data.dob=="undefined" || data.dob=='' || data.dob== null)?'0000-00-00':data.dob;  
    const passport_no=(typeof data.passport=="undefined")?'':data.passport;   
    const street=(typeof data.street=="undefined")?'':data.street;   
    const city=(typeof data.city=="undefined")?'':data.city; 
    const state=(typeof data.state=="undefined")?'':data.state;  	
    const postal_code=(typeof data.postal_code=="undefined")?'':data.postal_code;    
    const tel_home=(typeof data.tel_home=="undefined")?'':data.tel_home;   
    const tel_work=(typeof data.tel_work=="undefined")?'':data.tel_work;   
    const cell_phone=(typeof data.cell_phone=="undefined")?'':data.cell_phone;    
    const email_address=(typeof data.email_address=="undefined")?'':data.email_address;  	  
    const country=(typeof data.country=="undefined")?'':data.country;   
    const qualification=(typeof data.qualification=="undefined")?'':data.qualification;  
    const reg_no=(typeof data.reg_no=="undefined")?'':data.reg_no;  	   	
    const diver_type=(typeof data.diver_type=="undefined")?'':data.diver_type;   
    const doctor_name=(typeof data.doctor_name=="undefined")?'':data.doctor_name;   
    const doctor_contact=(typeof data.doctor_contact=="undefined")?'':data.doctor_contact;     

    let medicalFit = '';
    const medical_fit = (typeof data.medical_fit === "undefined") ? '' : data.medical_fit;  	 
    if (medical_fit === 'Other') {
      medicalFit = (typeof data.medicalFitInputValue === "undefined") ? '' : data.medicalFitInputValue;
    } else {
      medicalFit = medical_fit;
    }

    let medicalRestriction = '';
    const medical_restriction = (typeof data.medical_restriction === "undefined") ? '' : data.medical_restriction;  	 
    if (medical_restriction === 'Other') {
      medicalRestriction = (typeof data.medicalRestrictionInputValue === "undefined") ? '' : data.medicalRestrictionInputValue;
    } else {
      medicalRestriction = medical_restriction;
    }

    let caseDcs = '';
    const case_dcs = (typeof data.case_dcs === "undefined") ? '' : data.case_dcs;  	 
    if (case_dcs === 'Other') {
      caseDcs = (typeof data.caseDcsInputValue === "undefined") ? '' : data.caseDcsInputValue;
    } else {
      caseDcs = case_dcs;            
    }
    const medication=(typeof data.medication=="undefined")?'':data.medication;    
    const chronic_ill=(typeof data.chronic_ill=="undefined")?'':data.chronic_ill;  
    const aid=(typeof data.aid=="undefined")?'':data.aid;  
    const aid_no=(typeof data.aid_no=="undefined")?'':data.aid_no; 
    const emer_name=(typeof data.emer_name=="undefined")?'':data.emer_name;    	
    const emer_phone=(typeof data.emer_phone=="undefined")?'':data.emer_phone;    	
    const kin_title=(typeof data.kin_title=="undefined")?'':data.kin_title;  	
    const kin_fname=(typeof data.kin_fname=="undefined")?'':data.kin_fname;  	  
    const kin_sname=(typeof data.kin_sname=="undefined")?'':data.kin_sname;  
    const kin_dob=(typeof data.kin_dob=="undefined" || data.kin_dob=='' || data.kin_dob== null)?'0000-00-00':data.kin_dob;  
    const kin_passport=(typeof data.kin_passport=="undefined")?'':data.kin_passport;  
    const kin_street=(typeof data.kin_street=="undefined")?'':data.kin_street;  
    const kin_city=(typeof data.kin_city=="undefined")?'':data.kin_city;  
    const kin_state=(typeof data.kin_state=="undefined")?'':data.kin_state;  
    const kin_postal_code=(typeof data.kin_postal_code=="undefined")?'':data.kin_postal_code;  	
    const kin_home=(typeof data.kin_home=="undefined")?'':data.kin_home;  
    const kin_work=(typeof data.kin_work=="undefined")?'':data.kin_work;    
    const kin_mobile=(typeof data.kin_mobile=="undefined")?'':data.kin_mobile;  
    const kin_email_address=(typeof data.kin_email_address=="undefined")?'':data.kin_email_address; 
    const kin_relation=(typeof data.kin_relation=="undefined")?'':data.kin_relation;        	
    const start_date=(typeof data.start_date=="undefined" || data.start_date=='' || data.start_date== null)?'0000:00:00':data.start_date;    	 	
    const payment_opt=(typeof data.payment_opt=="undefined")?'':data.payment_opt;      
    const new_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6);           	
    const choose_membership='commercial_professional';   
    const package_name='professional';   	   


    let memberPrefix = ''; // Initialize member prefix

      if (package_name === 'individual') {
        memberPrefix = 'DANCI';
      } else if (package_name === 'professional') {
          memberPrefix = 'DANCP';
      } else if (package_name === 'company') {
          memberPrefix = 'DANCC';
      }else if (package_name === 'student') {
        memberPrefix = 'DANCS';
      }
    
    const memberNO = `${memberPrefix} ${randomNo}`; // Generate random number and the determined prefix

    // Check if the email already exists in the database
    const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
    const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${email_address}'`;
    const existingUser = await queryDatabase(emailCheckQuery, emailStream);

    if (existingUser.length > 0) {        
	  return res.status(500).json({ error: 'Email already exist','error_field':'email_address' });	  
    }
    else{  	    

	  const sqlQuery = `INSERT INTO membership (title1,fname1,sname1,dob1,passport1,address1,city1,province1,postcode1,telephone1,telephone2,cellphone1,email1,country1,qualification,registration_no,diver_type,doctor_name,doctor_phone,medical_fit,medical_restriction,case_dcs,medical_aid,medical_aid_no,medication,chronic_ill,contact_person,contact_phone,title2,fname2,sname2,dob2,passport2,address3,city2,province2,postcode2,telephone3,telephone4,cellphone2,email2,relation,startdate,pay_method,choose_membership,package_name,password,member_no,package_for,verifyEmail,verifyAmount,status1) 
	  VALUES ('${title}','${first_name}','${last_name}','${dob}','${passport_no}','${street}','${city}','${state}','${postal_code}','${tel_home}','${tel_work}','${cell_phone}','${email_address}','${country}','${qualification}','${reg_no}','${diver_type}','${doctor_name}','${doctor_contact}','${medicalFit}','${medicalRestriction}','${caseDcs}','${aid}','${aid_no}','${medication}','${chronic_ill}','${emer_name}','${emer_phone}','${kin_title}','${kin_fname}','${kin_sname}','${kin_dob}','${kin_passport}','${kin_street}','${kin_city}','${kin_state}','${kin_postal_code}','${kin_home}','${kin_work}','${kin_mobile}','${kin_email_address}','${kin_relation}','${start_date}','${payment_opt}','${choose_membership}','${package_name}','${new_pwd}','${memberNO}','year','verified','not_paid','INACTIVE')`;      	
	  
	  await queryDatabase(sqlQuery,stream);   				  		 		

		sendForgotEmail(email_address, new_pwd);	 
		res.status(200).json({ message: 'Members registered successfully. Check your email for verification.' });  
    }
	 
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error, error:error.message });
  } 
});
app.post('/commercial_student_register', async (req, res) => {            
  const data = req.body;                                           

  try {
    // Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    const randomNo = Math.floor(Math.random() * 1000); // Generate a random number
    
    // Insert data into the database  
    const school_name=(typeof data.school_name=="undefined")?'':data.school_name;  
    const school_phone=(typeof data.school_phone=="undefined")?'':data.school_phone;  	
    const first_name=(typeof data.first_name=="undefined")?'':data.first_name;    
    const last_name=(typeof data.last_name=="undefined")?'':data.last_name;  	
    const email_address=(typeof data.email_address=="undefined")?'':data.email_address;  	  
    const website=(typeof data.website=="undefined")?'':data.website;   
    const street=(typeof data.street=="undefined")?'':data.street;   
    const city=(typeof data.city=="undefined")?'':data.city; 
    const state=(typeof data.state=="undefined")?'':data.state;  	
    const postal_code=(typeof data.postal_code=="undefined")?'':data.postal_code; 
    const start_date=(typeof data.start_date=="undefined" || data.start_date=='' || data.start_date== null)?'0000:00:00':data.start_date;    	 	
    const payment_opt=(typeof data.payment_opt=="undefined")?'':data.payment_opt; 
    const new_pwd=generateRandomPassword(Math.floor(Math.random() * 2) + 6);;  
    const choose_membership='commercial_student';   	   
    const package_name='student';   	    

    let memberPrefix = '';
      if (package_name === 'individual') {
        memberPrefix = 'DANCI';
      } else if (package_name === 'professional') {
          memberPrefix = 'DANCP';
      } else if (package_name === 'company') {
          memberPrefix = 'DANCC';
      }else if (package_name === 'student') {
        memberPrefix = 'DANCS';
      }
    
    const memberNO = `${memberPrefix} ${randomNo}`;

    // Check if the email already exists in the database
    const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
    const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${email_address}'`;
    const existingUser = await queryDatabase(emailCheckQuery, emailStream);

    if (existingUser.length > 0) {        
	  return res.status(500).json({ error: 'Email already exist','error_field':'email_address' });	  
    }
    else{  	    

	  const sqlQuery = `INSERT INTO membership (company_name,contact_phone,fname1,sname1,address1,city1,province1,postcode1,email1,verifyEmail,verifyAmount,status1,choose_membership,package_name,package_for,member_no,password,address2,startdate,pay_method) VALUES ('${school_name}','${school_phone}','${first_name}','${last_name}','${street}','${city}','${state}','${postal_code}','${email_address}','verified','not_paid','INACTIVE','${choose_membership}','${package_name}','year','${memberNO}','${new_pwd}','${website}','${start_date}','${payment_opt}')`;      	
	  
	  const result = await queryDatabase(sqlQuery,stream);   				  		 		
	  const insert_id = result.insertId; 
	  
	   if(insert_id >0)			   
		{
			// Send a success response	 			
			for(var i=1;i<=5;i++)
			{ 								
				const fname=(typeof data[`fname${i}`]=="undefined")?'':data[`fname${i}`];  
				const sname=(typeof data[`sname${i}`]=="undefined")?'':data[`sname${i}`];  
				const dob=(typeof data[`dob${i}`]=="undefined" || data[`dob${i}`]=='' || data[`dob${i}`]== null)?'0000-00-00':data[`dob${i}`];  
				const passport=(typeof data[`passport${i}`]=="undefined")?'':data[`passport${i}`]; 
				const nationality=(typeof data[`nationality${i}`]=="undefined")?'':data[`nationality${i}`];  
				const period=(typeof data[`period${i}`]=="undefined")?'':data[`period${i}`];  
				const medication=(typeof data[`medication${i}`]=="undefined")?'':data[`medication${i}`];  
				const chronic_ill=(typeof data[`chronic_ill${i}`]=="undefined")?'':data[`chronic_ill${i}`]; 
				const aid=(typeof data[`aid${i}`]=="undefined")?'':data[`aid${i}`];   
				const aid_no=(typeof data[`aid_no${i}`]=="undefined")?'':data[`aid_no${i}`];   
				const emer_name=(typeof data[`emer_name${i}`]=="undefined")?'':data[`emer_name${i}`];   
				const emer_phone=(typeof data[`emer_phone${i}`]=="undefined")?'':data[`emer_phone${i}`];   
        const choose_membership='com_student'; 

        const randomNo2 = Math.floor(Math.random() * 1000);
        const package_name='student';   	    

        let memberPrefix2 = '';
          if (package_name === 'individual') {
            memberPrefix2 = 'DANCI';
          } else if (package_name === 'professional') {
              memberPrefix2 = 'DANCP';
          } else if (package_name === 'company') {
              memberPrefix2 = 'DANCC';
          }else if (package_name === 'student') {
            memberPrefix2 = 'DANCS';
          }
        
        const memberNO2 = `${memberPrefix2} ${randomNo2}`;

				const stream = await createSshTunnel(sshTunnelConfig,dbServer);  

				if(fname !='' || sname !='' || dob !='0000-00-00' || passport !='' || nationality !='' || period !='' || medication !='' || chronic_ill !='' || aid !='' || aid_no !='' || emer_name !='' ||  emer_phone !='')
				{
					const sqlQuery = `INSERT INTO membership (parent_id,fname1,sname1,dob1,passport1,country1,package_for, medication,chronic_ill,medical_aid,medical_aid_no,contact_person,contact_phone,choose_membership,verifyEmail,verifyAmount,status1,member_no,package_name) 
					VALUES ('${insert_id}','${fname}','${sname}','${dob}','${passport}','${nationality}','${period}','${medication}','${chronic_ill}','${aid}','${aid_no}','${emer_name}','${emer_phone}', '${choose_membership}','verified','paid','ACTIVE','${memberNO2}','${package_name}')`;               
				
					await queryDatabase(sqlQuery,stream); 			  
				}
				  				
			} 	 	
		  sendForgotEmail(email_address, new_pwd);	 
			res.status(200).json({ message: 'Members registered successfully. Check your email for verification.' });  
		} 
		else
		{
			res.status(500).json({ error: 'Record could not inserted!' });  	            
		} 	 
		
      
    }
	 
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error });
  } 
});

app.post('/commercial_company_register', async (req, res) => {            
  const data = req.body;                                           

  try {
    // Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);
    const randomNo = Math.floor(Math.random() * 1000); // Generate a random number
    
    // Insert data into the database  
    const company_name=(typeof data.company_name=="undefined")?'':data.company_name;  
    const company_regNo=(typeof data.company_regNo=="undefined")?'':data.company_regNo;  	
    const vat_regNo=(typeof data.vat_regNo=="undefined")?'':data.vat_regNo;    
    const first_name=(typeof data.first_name=="undefined")?'':data.first_name;  	
    const last_name=(typeof data.last_name=="undefined")?'':data.last_name;  	
    const contact_phone=(typeof data.contact_phone=="undefined")?'':data.contact_phone;  	
    const email_address=(typeof data.email_address=="undefined")?'':data.email_address;  	  
    const website=(typeof data.website=="undefined")?'':data.website;   
    const street=(typeof data.street=="undefined")?'':data.street;   
    const city=(typeof data.city=="undefined")?'':data.city; 
    const state=(typeof data.state=="undefined")?'':data.state;  	
    const postal_code=(typeof data.postal_code=="undefined")?'':data.postal_code; 
    const start_date=(typeof data.start_date=="undefined" || data.start_date=='' || data.start_date== null)?'0000:00:00':data.start_date;    	 	
    const payment_opt=(typeof data.payment_opt=="undefined")?'':data.payment_opt; 
    const new_pwd=generateRandomPassword(Math.floor(Math.random() * 2) + 6);;  
    const choose_membership='commercial_company';   	   
    const package_name='company';   	    

    let memberPrefix = '';
      if (package_name === 'individual') {
        memberPrefix = 'DANCI';
      } else if (package_name === 'professional') {
          memberPrefix = 'DANCP';
      } else if (package_name === 'company') {
          memberPrefix = 'DANCC';
      }else if (package_name === 'student') {
        memberPrefix = 'DANCS';
      }
    
    const memberNO = `${memberPrefix} ${randomNo}`;

    // Check if the email already exists in the database
    const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
    const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${email_address}'`;
    const existingUser = await queryDatabase(emailCheckQuery, emailStream);

    if (existingUser.length > 0) {        
	  return res.status(500).json({ error: 'Email already exist','error_field':'email_address' });	  
    }
    else{  	    

	  const sqlQuery = `INSERT INTO membership (company_name,contact_phone,registration_no,fname1,sname1,address1,city1,province1,postcode1,email1,verifyEmail,verifyAmount,status1,choose_membership,package_name,package_for,member_no,password,address2,startdate,pay_method,cellphone1) VALUES ('${company_name}','${company_regNo}','${vat_regNo}','${first_name}','${last_name}','${street}','${city}','${state}','${postal_code}','${email_address}','verified','not_paid','INACTIVE','${choose_membership}','${package_name}','year','${memberNO}','${new_pwd}','${website}','${start_date}','${payment_opt}','${contact_phone}')`;      	
	  
	  const result = await queryDatabase(sqlQuery,stream);   				  		 		
	  const insert_id = result.insertId; 
	  
	   if(insert_id >0)			   
		{
			// Send a success response	 			
			for(var i=1;i<=5;i++)
			{ 								
				const staff_fname=(typeof data[`staff_fname${i}`]=="undefined")?'':data[`staff_fname${i}`];  
				const staff_sname=(typeof data[`staff_sname${i}`]=="undefined")?'':data[`staff_sname${i}`];  
				const staff_dob=(typeof data[`staff_dob${i}`]=="undefined" || data[`staff_dob${i}`]=='' || data[`staff_dob${i}`]== null)?'0000-00-00':data[`staff_dob${i}`];  
				const staff_passport=(typeof data[`staff_passport${i}`]=="undefined")?'':data[`staff_passport${i}`]; 
				const glass_type=(typeof data[`glass_type${i}`]=="undefined")?'':data[`glass_type${i}`]; 
				const staff_regNo=(typeof data[`staff_regNo${i}`]=="undefined")?'':data[`staff_regNo${i}`]; 
				const diver_type=(typeof data[`diver_type${i}`]=="undefined")?'':data[`diver_type${i}`]; 
				const nationality=(typeof data[`nationality${i}`]=="undefined")?'':data[`nationality${i}`];  
				const doctor_name=(typeof data[`doctor_name${i}`]=="undefined")?'':data[`doctor_name${i}`]; 
				const doctor_phone=(typeof data[`doctor_phone${i}`]=="undefined")?'':data[`doctor_phone${i}`];  

        let medicalFit = '';
        const medical_fit = (typeof data[`medical_fit${i}`]=="undefined")?'':data[`medical_fit${i}`];  	 
        if (medical_fit === 'Other') {
          medicalFit = (typeof data[`medicalFitValue${i}`]=="undefined")?'':data[`medicalFitValue${i}`];
        } else {
          medicalFit = medical_fit;
        }
    
        let medicalRestriction = '';
        const medical_restriction = (typeof data[`restrictions${i}`]=="undefined")?'':data[`restrictions${i}`];  	 
        if (medical_restriction === 'Other') {
          medicalRestriction = (typeof data[`restrictionsValue${i}`]=="undefined")?'':data[`restrictionsValue${i}`];
        } else {
          medicalRestriction = medical_restriction;
        }
    
        let caseDcs = '';
        const case_dcs = (typeof data[`staff_dcs${i}`]=="undefined")?'':data[`staff_dcs${i}`];  	 
        if (case_dcs === 'Other') {
          caseDcs = (typeof data[`staffDcsValue${i}`]=="undefined")?'':data[`staffDcsValue${i}`];
        } else {
          caseDcs = case_dcs;
        }

				const staff_medication=(typeof data[`staff_medication${i}`]=="undefined")?'':data[`staff_medication${i}`];  
				const staff_chronic=(typeof data[`staff_chronic${i}`]=="undefined")?'':data[`staff_chronic${i}`]; 
				const staff_aid=(typeof data[`staff_aid${i}`]=="undefined")?'':data[`staff_aid${i}`];   
				const staff_aidNo=(typeof data[`staff_aidNo${i}`]=="undefined")?'':data[`staff_aidNo${i}`];   
				const emer_name=(typeof data[`emer_name${i}`]=="undefined")?'':data[`emer_name${i}`];   
				const emer_phone=(typeof data[`emer_phone${i}`]=="undefined")?'':data[`emer_phone${i}`];   
        const choose_membership='com_company'; 

        const randomNo2 = Math.floor(Math.random() * 1000);
        const package_name='company';   	    

        let memberPrefix2 = '';
          if (package_name === 'individual') {
            memberPrefix2 = 'DANCI';
          } else if (package_name === 'professional') {
              memberPrefix2 = 'DANCP';
          } else if (package_name === 'company') {
              memberPrefix2 = 'DANCC';
          }else if (package_name === 'student') {
            memberPrefix2 = 'DANCS';
          }
        
        const memberNO2 = `${memberPrefix2} ${randomNo2}`;

				const stream = await createSshTunnel(sshTunnelConfig,dbServer);  

				if(staff_fname !='' || staff_sname !='' || staff_dob !='0000-00-00' || staff_passport !='' || nationality !='' || glass_type !='' || staff_regNo !='' || diver_type !='' || doctor_name !='' || doctor_phone !='' || medicalFit !='' || medicalRestriction !=''  || caseDcs !='' || staff_medication !='' || staff_chronic !='' || staff_aid !='' || staff_aidNo !='' || emer_name !='' ||  emer_phone !='')
				{
					const sqlQuery = `INSERT INTO membership (parent_id,fname1,sname1,dob1,passport1,country1,glass_type,registration_no,diver_type,doctor_name,doctor_phone,medical_fit,medical_restriction,case_dcs,medication,chronic_ill,medical_aid,medical_aid_no,contact_person,contact_phone,member_no,choose_membership,verifyEmail,verifyAmount,status1,package_name) 
					VALUES ('${insert_id}','${staff_fname}','${staff_sname}','${staff_dob}','${staff_passport}','${nationality}','${glass_type}','${staff_regNo}','${diver_type}','${doctor_name}','${doctor_phone}','${medicalFit}','${medicalRestriction}','${caseDcs}','${staff_medication}','${staff_chronic}','${staff_aid}','${staff_aidNo}','${emer_name}','${emer_phone}','${memberNO2}','${choose_membership}','verified','paid','ACTIVE','${package_name}')`;               
				
					await queryDatabase(sqlQuery,stream); 			  
				}
				  				
			} 	 	
		  sendForgotEmail(email_address, new_pwd);	 
			res.status(200).json({ message: 'Members registered successfully. Check your email for verification.' });  
		} 
		else
		{
			res.status(500).json({ error: 'Record could not inserted!' });  	            
		} 	 
		
      
    }
	 
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error });
  } 
});
// register save data to membership table -end queries


// update membership package to DANM or DANX
app.post('/update_annual_package/:id', async (req, res) => { 
  const id = req.params.id;    
  const data = req.body;  

  const stream = await createSshTunnel(sshTunnelConfig, dbServer);	   
  const sqlQuery = `SELECT * FROM membership WHERE id=${id}`;	     	    
  const result = await queryDatabase(sqlQuery, stream);			
  
  if (!data.member_choose || data.member_choose === "") {
    return res.status(500).json({ error: 'Choose membership is required','error_field':'member_choose' });
  }else if (!data.pack_for || data.pack_for === "") {
    return res.status(500).json({ error: 'Package for is required','error_field':'pack_for' });
  }
  else if (!data.pay_type || data.pay_type === "") {
    return res.status(500).json({ error: 'Payment Type is required','error_field':'pay_type' });
  } 

  
  
  if(result.length==1)  
  {
	  try {  

		// Establish SSH tunnel
        const stream = await createSshTunnel(sshTunnelConfig, dbServer); 		
        const randomNo = Math.floor(Math.random() * 1000); // Generate a random number

		    const member_choose=(typeof data.member_choose=="undefined")?'':data.member_choose;  	
        const pack_for=(typeof data.pack_for=="undefined")?'':data.pack_for;    
        const price=(typeof data.price=="undefined")?'':data.price;  

        let memberPrefix = ''; // Initialize member prefix
        // Check package_name to determine the member prefix
        if (member_choose === 'Master Dive Pro') {
            memberPrefix = 'DANM';
        }else if (member_choose === 'Master Tech') {
          memberPrefix = 'DANX';
        }
        
        const memberNO = `${memberPrefix} ${randomNo}`;

		    const sqlQuery = `UPDATE membership SET package_name='${member_choose}',package_for='${pack_for}',package_price='${price}', member_no='${memberNO}' WHERE id = ${id}`; 
          
        await queryDatabase(sqlQuery,stream);  
		  
		res.status(200).json({ message: 'Annual member upgraded successfully' });     		       	    		 		
		 
	  } catch (error) {
		// Handle database or other errors 	  		
		console.error('Error adding data to the database:', error);
		return res.status(500).json({ error: error });				
	  }
	  //
  }
  else
  {
	  return res.status(500).json({ error:'Record not found!' });					
  }

  
})  
// end queries


// custom sql queries of Annual Dashboard
app.get('/main_member/:id', async (req, res) => { 
  const id = req.params.id;		 
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);		
    const sqlQuery = `SELECT * FROM membership WHERE id=${id} OR parent_id = ${id}`;
    // console.log(sqlQuery);
    const results = await queryDatabase(sqlQuery, stream);		  		
    res.status(200).json(results);   
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });		    	
  }
}) 

app.post('/update_annual_membership/:id', async (req, res) => { 
  const id = req.params.id; 
  const data = req.body;  

  const stream = await createSshTunnel(sshTunnelConfig, dbServer);	   
  const sqlQuery = `SELECT * FROM membership WHERE id=${id}`;	     	    
  const result = await queryDatabase(sqlQuery, stream);			
  
  if(result.length==1)  
  {
	  try {  

		// Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);   

		const per_title=(typeof data.per_title=="undefined" || data.per_title== null)?'':data.per_title;    	
		const first_name=(typeof data.first_name=="undefined" || data.first_name== null)?'':data.first_name;    	
		const surname=(typeof data.surname=="undefined" || data.surname== null)?'':data.surname;    	
		const per_dob=(typeof data.per_dob=="undefined" || data.per_dob== null)?'0000:00:00':data.per_dob;  	
		const per_passport=(typeof data.per_passport=="undefined" || data.per_passport== null)?'':data.per_passport;    	
		const per_address1=(typeof data.per_address1=="undefined" || data.per_address1== null)?'':data.per_address1;    	
		const per_address2=(typeof data.per_address2=="undefined" || data.per_address2== null)?'':data.per_address2;    	
		const per_city=(typeof data.per_city=="undefined" || data.per_city== null)?'':data.per_city;    	
		const per_province=(typeof data.per_province=="undefined" || data.per_province== null)?'':data.per_province;    	
		const post_code=(typeof data.post_code=="undefined" || data.post_code== null)?'':data.post_code;    	
		const per_country=(typeof data.per_country=="undefined" || data.per_country== null)?'':data.per_country;    	
		const tel_home=(typeof data.tel_home=="undefined" || data.tel_home== null)?'':data.tel_home;    	
		const tel_work=(typeof data.tel_work=="undefined" || data.tel_work== null)?'':data.tel_work;    	
		const cell_phone=(typeof data.cell_phone=="undefined" || data.cell_phone== null)?'':data.cell_phone;    	
		const fax=(typeof data.fax=="undefined" || data.fax== null)?'':data.fax;    	
		const medical_aid=(typeof data.medical_aid=="undefined" || data.medical_aid== null)?'':data.medical_aid;    	
		const medical_no=(typeof data.medical_no=="undefined" || data.medical_no== null)?'':data.medical_no;    	
		const medical_aid_dep=(typeof data.medical_aid_dep=="undefined" || data.medical_aid_dep== null)?'':data.medical_aid_dep;    	
		const refer_by=(typeof data.refer_by=="undefined" || data.refer_by== null)?'':data.refer_by;    	
		const agency=(typeof data.agency=="undefined" || data.agency== null)?'':data.agency;    	
    const spear_diver = (typeof data.spear_diver === 'undefined' || data.spear_diver === null || data.spear_diver === '') ? 'N/A' : data.spear_diver;
    const free_diver = (typeof data.free_diver === 'undefined' || data.free_diver === null || data.free_diver === '') ? 'N/A' : data.free_diver;
    const creational = (typeof data.creational === 'undefined' || data.creational === null || data.creational === '') ? 'No' : data.creational;
		const qualification=(typeof data.qualification=="undefined" || data.qualification== null)?'':data.qualification;    	
    const work_diver = (typeof data.work_diver === 'undefined' || data.work_diver === null || data.work_diver === '') ? 'N/A' : data.work_diver;
		const kin_title=(typeof data.kin_title=="undefined" || data.kin_title== null)?'':data.kin_title;    	
		const kin_name=(typeof data.kin_name=="undefined" || data.kin_name== null)?'':data.kin_name;    	
		const kin_surname=(typeof data.kin_surname=="undefined" || data.kin_surname== null)?'':data.kin_surname;    	
		const kin_dob=(typeof data.kin_dob=="undefined" || data.kin_dob== null)?'0000:00:00':data.kin_dob;  	
		const kin_passport=(typeof data.kin_passport=="undefined" || data.kin_passport== null)?'':data.kin_passport;    	
		const kin_address1=(typeof data.kin_address1=="undefined" || data.kin_address1== null)?'':data.kin_address1;    	
		const kin_address2=(typeof data.kin_address2=="undefined" || data.kin_address2== null)?'':data.kin_address2;    	
		const kin_city=(typeof data.kin_city=="undefined" || data.kin_city== null)?'':data.kin_city;    	
		const kin_province=(typeof data.kin_province=="undefined" || data.kin_province== null)?'':data.kin_province;    	
		const kin_postal=(typeof data.kin_postal=="undefined" || data.kin_postal== null)?'':data.kin_postal;    	
		const kin_country=(typeof data.kin_country=="undefined" || data.kin_country== null)?'':data.kin_country;    	
		const kin_home=(typeof data.kin_home=="undefined" || data.kin_home== null)?'':data.kin_home;    	
		const kin_work=(typeof data.kin_work=="undefined" || data.kin_work== null)?'':data.kin_work;    	
		const kin_cell=(typeof data.kin_cell=="undefined" || data.kin_cell== null)?'':data.kin_cell;  
		const kin_email=(typeof data.kin_email=="undefined" || data.kin_email== null)?'':data.kin_email;    	
		const kin_relation=(typeof data.kin_relation=="undefined" || data.kin_relation== null)?'':data.kin_relation;    
	
		const sqlQuery = `UPDATE membership SET title1='${per_title}',fname1='${first_name}',sname1='${surname}',passport1='${per_passport}', address1='${per_address1}',address2='${per_address2}',city1='${per_city}',province1='${per_province}',postcode1='${post_code}',country1='${per_country}',telephone1='${tel_home}',telephone2='${tel_work}',cellphone1='${cell_phone}',fax='${fax}', dob1='${per_dob}', medical_aid='${medical_aid}', medical_aid_no='${medical_no}', aid_dependent='${medical_aid_dep}', refer_by='${refer_by}', agency='${agency}', qualification='${qualification}', spear_diver='${spear_diver}', free_diver='${free_diver}', creational='${creational}', work_diver='${work_diver}', title2='${kin_title}',fname2='${kin_name}',sname2='${kin_surname}',passport2='${kin_passport}', address3='${kin_address1}',address4='${kin_address2}',city2='${kin_city}',province2='${kin_province}',postcode2='${kin_postal}',country2='${kin_country}',telephone3='${kin_home}',telephone4='${kin_work}',cellphone2='${kin_cell}',email2='${kin_email}',relation='${kin_relation}', dob2='${kin_dob}' WHERE id=${id}`;   		
					 
		await queryDatabase(sqlQuery,stream);      		       
		
		// Send a success response	  		
		res.status(200).json({ message: 'Annual membership details saved.' }); 
      	    		 		
	  } catch (error) {
		// Handle database or other errors 	  		
		console.error('Error adding data to the database:', error);
		return res.status(500).json({ error: error });				
	  }
  }
  else
  {
	  return res.status(500).json({ error:'Record not found!' });					
  }
});

app.post('/add_diving', async (req, res) => {			
  const data = req.body;		
  const stream = await createSshTunnel(sshTunnelConfig, dbServer);	    
  const sqlQuery = 'SELECT (IFNULL(MAX(id),0)+1) AS max_id FROM membership';	   	  
  const result = await queryDatabase(sqlQuery, stream);	   

  try {
    // Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);  	
         
      const row_id=(typeof data.parent_id=="undefined")?'':data.parent_id;  
      const member_type=(typeof data.member_type=="undefined")?'':data.member_type;    
      const dive_title=(typeof data.dive_title=="undefined")?'':data.dive_title;  	
			const dive_fname=(typeof data.dive_fname=="undefined")?'':data.dive_fname;    
			const dive_surname=(typeof data.dive_surname=="undefined")?'':data.dive_surname;  
			const dive_dob=(typeof data.dive_dob=="undefined" || data.dive_dob== null)?'0000:00:00':data.dive_dob;    	
			const dive_passport=(typeof data.dive_passport=="undefined")?'':data.dive_passport;  	
			const dive_home=(typeof data.dive_home=="undefined")?'':data.dive_home;  	
			const dive_work=(typeof data.dive_work=="undefined")?'':data.dive_work;      	
			const dive_cell=(typeof data.dive_cell=="undefined")?'':data.dive_cell;      
			const dive_email=(typeof data.dive_email=="undefined")?'':data.dive_email;        	
			const dive_relation=(typeof data.dive_relation=="undefined")?'':data.dive_relation;    
			const dive_qualification=(typeof data.dive_qualification=="undefined")?'':data.dive_qualification;     	  
			const dive_agency=(typeof data.dive_agency=="undefined")?'':data.dive_agency;  	
      const dive_workingAs = (typeof data.dive_workingAs === 'undefined' || data.dive_workingAs === null || data.dive_workingAs === '') ? 'No' : data.dive_workingAs;
      const dive_free_spear = (typeof data.dive_free_spear === 'undefined' || data.dive_free_spear === null || data.dive_free_spear === '') ? 'N/A' : data.dive_free_spear;
      const new_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6);
	
      const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
      const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${dive_email}'`;
      const existingUser = await queryDatabase(emailCheckQuery, emailStream);

      if (existingUser.length > 0) {
        return res.status(500).json({ message: 'Email already exists' });
      }else{
        const sqlQuery = `INSERT INTO membership (parent_id,membertype,title1,fname1,sname1,passport1,member_no,relation,telephone1,telephone2,cellphone1,email1,qualification,agency,status1,free_diver,dob1,working_as,verifyEmail,verifyAmount,password,choose_membership)  VALUES ('${row_id}','${member_type}','${dive_title}','${dive_fname}','${dive_surname}','${dive_passport}','','${dive_relation}','${dive_home}','${dive_work}','${dive_cell}','${dive_email}','${dive_qualification}','${dive_agency}','INACTIVE','${dive_free_spear}','${dive_dob}','${dive_workingAs}','verified','not_paid','${new_pwd}','annual')`;  
      
        await queryDatabase(sqlQuery,stream);   	        

        // Send a success response
        res.status(200).json({ message: 'Family member added successfully' });   					  		
      }
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error });
  }
});  

app.post('/update_diving/:id', async (req, res) => { 
  const id = req.params.id;   
  const data = req.body;  

  const stream = await createSshTunnel(sshTunnelConfig, dbServer);	   
  const sqlQuery = `SELECT * FROM membership WHERE id=${id}`;	     	    
  const result = await queryDatabase(sqlQuery, stream);	   

  if(result.length==1)  
  {
	  try {  
			 const stream = await createSshTunnel(sshTunnelConfig, dbServer);	    			
			 const row_id=(typeof data.parent_id=="undefined")?'':data.parent_id;  
        const member_type=(typeof data.member_type=="undefined")?'':data.member_type;    
        const dive_title=(typeof data.dive_title=="undefined")?'':data.dive_title;  	
        const dive_fname=(typeof data.dive_fname=="undefined")?'':data.dive_fname;    
        const dive_surname=(typeof data.dive_surname=="undefined")?'':data.dive_surname;  
        const dive_dob=(typeof data.dive_dob=="undefined" || data.dive_dob== null)?'0000:00:00':data.dive_dob;    	
        const dive_passport=(typeof data.dive_passport=="undefined")?'':data.dive_passport;  	
        const dive_home=(typeof data.dive_home=="undefined")?'':data.dive_home;  	
        const dive_work=(typeof data.dive_work=="undefined")?'':data.dive_work;      	
        const dive_cell=(typeof data.dive_cell=="undefined")?'':data.dive_cell;      
        const dive_email=(typeof data.dive_email=="undefined")?'':data.dive_email;        	
        const dive_relation=(typeof data.dive_relation=="undefined")?'':data.dive_relation;    
        const dive_qualification=(typeof data.dive_qualification=="undefined")?'':data.dive_qualification;     	  
        const dive_agency=(typeof data.dive_agency=="undefined")?'':data.dive_agency;  	
        const dive_workingAs = (typeof data.dive_workingAs === 'undefined' || data.dive_workingAs === null || data.dive_workingAs === '') ? 'No' : data.dive_workingAs;
        const dive_free_spear = (typeof data.dive_free_spear === 'undefined' || data.dive_free_spear === null || data.dive_free_spear === '') ? 'N/A' : data.dive_free_spear;
			     

      const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
      const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${dive_email}' AND id != ${id}`;
      const existingUser = await queryDatabase(emailCheckQuery, emailStream);

      if (existingUser.length > 0) {
        return res.status(500).json({ message: 'Email already exists' });
      }else{
        
        const sqlQuery = `UPDATE membership SET title1='${dive_title}',fname1='${dive_fname}',sname1='${dive_surname}',passport1='${dive_passport}',relation='${dive_relation}',telephone1='${dive_home}',telephone2='${dive_work}',cellphone1='${dive_cell}',email1='${dive_email}',qualification='${dive_qualification}',agency='${dive_agency}',free_diver='${dive_free_spear}',dob1='${dive_dob}',working_as='${dive_workingAs}' WHERE id=${id}`;   				
        
        await queryDatabase(sqlQuery,stream);   			 			       
        res.status(200).json({ message: 'Family member updated successfully' });     
      }
		} catch (error) {   
		// Handle database or other errors 	  		
		console.error('Error adding data to the database:', error);
		return res.status(500).json({ error: error });				
	  } 	  
  }
  else
  {
	  return res.status(500).json({ error:'Record not found!' });		  				
  }   	 		  
  
}) 

app.get('/fetch_stud_annual/:parent_id', async (req, res) => { 
  const parent_id = req.params.parent_id;		 
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);		
    const sqlQuery = `SELECT * FROM membership WHERE parent_id=${parent_id} AND membertype='Entry-Level Student' AND choose_membership='annual'`;	         	    
    const results = await queryDatabase(sqlQuery, stream);		  		
    res.status(200).json(results);   
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });		    	
  }
}) 
app.post('/save_stud_annual', async (req, res) => {			
  const data = req.body;		
  const stream = await createSshTunnel(sshTunnelConfig, dbServer);	    
  const sqlQuery = 'SELECT (IFNULL(MAX(id),0)+1) AS max_id FROM membership';	   	  
  const result = await queryDatabase(sqlQuery, stream);	       

  try {
    // Establish SSH tunnel
      const stream = await createSshTunnel(sshTunnelConfig, dbServer);  
      const randomNo = Math.floor(Math.random() * 1000); 
      const memberNO = `DANT ${randomNo}`;
         
      const parent_id=(typeof data.parent_id=="undefined")?'':data.parent_id;    
      const studAnnual_title=(typeof data.per_title=="undefined")?'':data.per_title;  	
			const studAnnual_fname=(typeof data.first_name=="undefined")?'':data.first_name;    
			const studAnnual_sname=(typeof data.surname=="undefined")?'':data.surname;  
			const studAnnual_dob=(typeof data.per_dob=="undefined" || data.per_dob== null)?'0000:00:00':data.per_dob;    	
			const studAnnual_passport=(typeof data.per_passport=="undefined" || data.per_passport== null)?'':data.per_passport;  
			const studAnnual_address1=(typeof data.per_address1=="undefined")?'':data.per_address1;  	
			const studAnnual_address2=(typeof data.per_address2=="undefined" || data.per_address2== null)?'':data.per_address2;  	
			const studAnnual_city=(typeof data.per_city=="undefined")?'':data.per_city;  	
			const studAnnual_province=(typeof data.per_province=="undefined")?'':data.per_province;  	
			const studAnnual_post=(typeof data.post_code=="undefined")?'':data.post_code;  	
			const studAnnual_country=(typeof data.per_country=="undefined")?'':data.per_country; 
			const studAnnual_home=(typeof data.tel_home=="undefined" || data.tel_home== null)?'':data.tel_home;  	    	
			const studAnnual_cellphone=(typeof data.cell_phone=="undefined")?'':data.cell_phone;      
			const studAnnual_email=(typeof data.per_email=="undefined")?'':data.per_email;        	
			const studAnnual_aid=(typeof data.medical_aid=="undefined")?'':data.medical_aid;    
			const studAnnual_aid_no=(typeof data.medical_no=="undefined")?'':data.medical_no;     	  
      const studAnnual_course=(typeof data.course_date=="undefined" || data.course_date== null)?'0000:00:00':data.course_date; 
	
      const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
      const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${studAnnual_email}'`;
      const existingUser = await queryDatabase(emailCheckQuery, emailStream);

      if (existingUser.length > 0) {
        return res.status(500).json({ message: 'Email already exists' });
      }
      else{
        const sqlQuery = `INSERT INTO membership (parent_id,title1,fname1,sname1,dob1,passport1,medical_aid,medical_aid_no,startdate,address1,address2,city1,province1,postcode1,country1,telephone1,cellphone1,email1,member_no,verifyEmail,verifyAmount,choose_membership,membertype,status1) VALUES ('${parent_id}','${studAnnual_title}','${studAnnual_fname}','${studAnnual_sname}','${studAnnual_dob}','${studAnnual_passport}','${studAnnual_aid}','${studAnnual_aid_no}','${studAnnual_course}','${studAnnual_address1}','${studAnnual_address2}','${studAnnual_city}','${studAnnual_province}','${studAnnual_post}','${studAnnual_country}','${studAnnual_home}','${studAnnual_cellphone}','${studAnnual_email}','${memberNO}','not_verified','not_paid','annual','Entry-Level Student','INACTIVE')`;  
        
        await queryDatabase(sqlQuery,stream);   	        

        // Send a success response
        res.status(200).json({ message: 'Student member added successfully.' });   					  		
      }
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error });
  }
}); 
app.post('/update_stud_annual/:id', async (req, res) => {			
  const id = req.params.id; 
  const data = req.body;  
  
  const stream = await createSshTunnel(sshTunnelConfig, dbServer);	 
  const sqlQuery = `SELECT * FROM membership WHERE id=${id}`;	     	    
  const result = await queryDatabase(sqlQuery, stream);	       

  if(result.length==1){
    try {
      // Establish SSH tunnel
        const stream = await createSshTunnel(sshTunnelConfig, dbServer);  	 
            
        const studAnnual_title=(typeof data.per_title=="undefined")?'':data.per_title;  	
			const studAnnual_fname=(typeof data.first_name=="undefined")?'':data.first_name;    
			const studAnnual_sname=(typeof data.surname=="undefined")?'':data.surname;  
			const studAnnual_dob=(typeof data.per_dob=="undefined" || data.per_dob== null)?'0000:00:00':data.per_dob;    	
			const studAnnual_passport=(typeof data.per_passport=="undefined")?'':data.per_passport;  
			const studAnnual_address1=(typeof data.per_address1=="undefined")?'':data.per_address1;  	
			const studAnnual_address2=(typeof data.per_address2=="undefined")?'':data.per_address2;  	
			const studAnnual_city=(typeof data.per_city=="undefined")?'':data.per_city;  	
			const studAnnual_province=(typeof data.per_province=="undefined")?'':data.per_province;  	
			const studAnnual_post=(typeof data.post_code=="undefined")?'':data.post_code;  	
			const studAnnual_country=(typeof data.per_country=="undefined")?'':data.per_country; 
			const studAnnual_home=(typeof data.tel_home=="undefined")?'':data.tel_home;  	    	
			const studAnnual_cellphone=(typeof data.cell_phone=="undefined")?'':data.cell_phone;      
			const studAnnual_email=(typeof data.per_email=="undefined")?'':data.per_email;        	
			const studAnnual_aid=(typeof data.medical_aid=="undefined")?'':data.medical_aid;    
			const studAnnual_aid_no=(typeof data.medical_no=="undefined")?'':data.medical_no;     	  
      const studAnnual_course=(typeof data.course_date=="undefined" || data.course_date== null)?'0000:00:00':data.course_date; 
    
      const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
      const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${studAnnual_email}' AND id != ${id}`;
      const existingUser = await queryDatabase(emailCheckQuery, emailStream);

        if (existingUser.length > 0) {
          return res.status(500).json({ message: 'Email already exists' });
        }
        else{

          const sqlQuery = `UPDATE membership SET title1='${studAnnual_title}',fname1='${studAnnual_fname}',sname1='${studAnnual_sname}',dob1='${studAnnual_dob}',passport1='${studAnnual_passport}',medical_aid='${studAnnual_aid}',medical_aid_no='${studAnnual_aid_no}',startdate='${studAnnual_course}',address1='${studAnnual_address1}',address2='${studAnnual_address2}',city1='${studAnnual_city}',province1='${studAnnual_province}',postcode1='${studAnnual_post}',country1= '${studAnnual_country}',telephone1='${studAnnual_home}',cellphone1='${studAnnual_cellphone}',email1='${studAnnual_email}' WHERE id = ${id}`; 
          
          await queryDatabase(sqlQuery,stream);   	        

          // Send a success response
          res.status(200).json({ message: 'Student member updated successfully' });   					  		
        }
    } catch (error) {
      // Handle database or other errors 	  		
      console.error('Error adding data to the database:', error);
      res.status(500).json({ error: error });
    }
  }else{
      return res.status(500).json({ error:'Record not found!' });		  				
  }
}); 

app.post('/update_stud_facility/:id', async (req, res) => {			
  const id = req.params.id; 
  const data = req.body;  
  
  const stream = await createSshTunnel(sshTunnelConfig, dbServer);	 
  const sqlQuery = `SELECT * FROM membership WHERE id=${id}`;	     	    
  const result = await queryDatabase(sqlQuery, stream);	       

  if(result.length==1){
    try {
      // Establish SSH tunnel
        const stream = await createSshTunnel(sshTunnelConfig, dbServer);  	 
            
        const ins_option=(typeof data.ins_option=="undefined")?'':data.ins_option;
        const fac_others=(typeof data.fac_others=="undefined")?'':data.fac_others;  	

			
        let facility;
        if (ins_option === "Associated Facility") {
            if(fac_others == ''){
              facility = "Independent Instructor";
            }else{
              facility = fac_others;
            }
        } else{
          facility = "Independent Instructor";
        }

        const sqlQuery = `UPDATE membership SET facility='${facility}' WHERE id = ${id}`; 
          
        await queryDatabase(sqlQuery,stream);   	        

        // Send a success response
        res.status(200).json({ message: 'Thank you, we are processing your application to join DAN-SA the worlds largest dive safety organisation.' });   					  		
        
    } catch (error) {
      // Handle database or other errors 	  		
      console.error('Error adding data to the database:', error);
      res.status(500).json({ error: error });
    }
  }else{
      return res.status(500).json({ error:'Record not found!' });		  				
  }
}); 


app.post('/save_industry_partner', async (req, res) => {			
  const data = req.body;	
  
  const indusrtyEmail=(typeof data.email_address=="undefined")?'':data.email_address;   
  const memberNo=(typeof data.member_no=="undefined")?'':data.member_no;  	    
  
  const stream = await createSshTunnel(sshTunnelConfig, dbServer);			     
  const sqlQuery = `SELECT * FROM membership WHERE email1='${indusrtyEmail}'`;	   			
  const chkresult = await queryDatabase(sqlQuery, stream);	  

  const steam = await createSshTunnel(sshTunnelConfig, dbServer);			     
  const sqlQuer = "SELECT (IFNULL(MAX(id),0)+1) AS max_id FROM membership";	   			
  const result = await queryDatabase(sqlQuer, steam);	 

  const next_id = (result.length>0)?result[0].max_id:0;   	

  /* const strem = await createSshTunnel(sshTunnelConfig, dbServer);			     
  const sqlQue="SELECT * FROM membership WHERE UPPER(choose_membership)='ANNUAL' AND UPPER(membertype)='INDUSTRY_PARTNER' "+`AND member_no=${memberNo}`;			  			
  const chkrsult = await queryDatabase(sqlQue, strem);	  */   
  
  if (!data.title || data.title === "") {
    return res.status(500).json({ error: 'Title is Missing','error_field':'title' });		
  } 
  else if (!data.first_name || data.first_name === "") {
    return res.status(500).json({ error: 'First name is Missing','error_field':'first_name' });		
  }   
  else if (!data.dob || data.dob === "") {
    return res.status(500).json({ error: 'Date of birth is Missing','error_field':'dob' });					
  }   
  else if (!data.cell_phone || data.cell_phone === "") {
    return res.status(500).json({ error: 'Cell phone is Missing','error_field':'cell_phone' });					
  }
  else if (!data.email_address || data.email_address === "") {
    return res.status(500).json({ error: 'Email is Missing','error_field':'email_address' });		
  }     
  else if (chkresult.length >0) {		
    return res.status(500).json({ error: 'Email already exist','error_field':'email_address' });							
  } 

  try {
		 // Establish SSH tunnel
		const stream = await createSshTunnel(sshTunnelConfig,dbServer); 	

		const parent_id=(typeof data.parent_id=="undefined")?'':data.parent_id;  	
		const title=(typeof data.title=="undefined")?'':data.title;  	
		const fname=(typeof data.first_name=="undefined")?'':data.first_name;    
		const sname=(typeof data.surname=="undefined")?'':data.surname;  	
		const dob=(typeof data.dob=="undefined" || data.dob== null)?'0000:00:00':data.dob;   
		const passport=(typeof data.passport=="undefined")?'':data.passport;  		
		const address1=(typeof data.address1=="undefined")?'':data.address1;  	
		const address2=(typeof data.address2=="undefined")?'':data.address2;  	
		const city=(typeof data.city=="undefined")?'':data.city;  	
		const province=(typeof data.province=="undefined")?'':data.province;   	
		const postal_code=(typeof data.post_code=="undefined")?'':data.post_code;  	
		const country=(typeof data.country=="undefined")?'':data.country;    
		const phonehome=(typeof data.tel_home=="undefined")?'':data.tel_home;  		
		const phonework=(typeof data.tel_work=="undefined")?'':data.tel_work;  
		const fax=(typeof data.fax=="undefined")?'':data.fax;  
		const cellphone=(typeof data.cell_phone=="undefined")?'':data.cell_phone;  	
		const email_address=(typeof data.email_address=="undefined")?'':data.email_address;  			 		
		const agency=(typeof data.agency=="undefined")?'':data.agency;  	
					
		const new_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6); // Generate a random password with length between 6 and 7 characters
			
		const sqlQuery = `INSERT INTO membership (title1,fname1,sname1,dob1,passport1,address1,address2,city1,province1,postcode1,country1,telephone1,telephone2,cellphone1,email1,password,member_no,verifyEmail,verifyAmount,agency,fax,status1,parent_id,choose_membership,membertype) VALUES 
		('${title}','${fname}','${sname}','${dob}','${passport}','${address1}','${address2}','${city}','${province}','${postal_code}','${country}','${phonehome}','${phonework}','${cellphone}','${email_address}','${new_pwd}','${memberNo}','verified','not_paid','${agency}','${fax}','ACTIVE','${parent_id}','annual','industry_partner')`;  		
				
		const result = await queryDatabase(sqlQuery,stream);    																		   		  
		const insert_id = result.insertId;    		  
		
		if(insert_id >0)	
		{
			// Send a success response	 		
			res.status(200).json({ message: 'Data added successfully' });   												   																		  			 		  	
		} 
		else
		{
			res.status(500).json({ error: 'Record could not inserted!' });  												            
		} 	  
      
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error });
  }
}); 

// Provider Student Course
app.get('/provider_courses', async (req, res) => {
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);		
    const sqlQuery = "SELECT * FROM courses where type='Provider'";	          
    const results = await queryDatabase(sqlQuery, stream);	  			
    res.status(200).json(results);
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });		    	
  }
}); 
app.post('/addProviderStudent', async (req, res) => {			
  const data = req.body;		
  const stream = await createSshTunnel(sshTunnelConfig, dbServer);	    
  const sqlQuery = 'SELECT (IFNULL(MAX(id),0)+1) AS max_id FROM membership';	   	  
  const result = await queryDatabase(sqlQuery, stream);	       

  try {
    // Establish SSH tunnel
      const stream = await createSshTunnel(sshTunnelConfig, dbServer);  
         
      const parent_id=(typeof data.parent_id=="undefined")?'':data.parent_id;    
      const per_title=(typeof data.per_title=="undefined" || data.per_title== null)?'':data.per_title;  	
			const first_name=(typeof data.first_name=="undefined")?'':data.first_name;    
			const surname=(typeof data.surname=="undefined")?'':data.surname;  
			const per_passport=(typeof data.per_passport=="undefined" || data.per_passport== null)?'':data.per_passport;  
			const per_address1=(typeof data.per_address1=="undefined")?'':data.per_address1;  	
			const per_address2=(typeof data.per_address2=="undefined" || data.per_address2== null)?'':data.per_address2;  	
			const per_city=(typeof data.per_city=="undefined")?'':data.per_city;  	
			const per_province=(typeof data.per_province=="undefined")?'':data.per_province;  	
			const post_code=(typeof data.post_code=="undefined")?'':data.post_code;  	
			const per_country=(typeof data.per_country=="undefined")?'':data.per_country; 	    	
			const cell_phone=(typeof data.cell_phone=="undefined")?'':data.cell_phone;      
			const per_email=(typeof data.per_email=="undefined")?'':data.per_email;        	
			const course_type=(typeof data.course_type=="undefined" || data.course_type== null)?'':data.course_type; 
      const course_loc=(typeof data.course_loc=="undefined" || data.course_loc== null)?'':data.course_loc;    
      const training_date=(typeof data.training_date=="undefined" || data.training_date== null)?'0000:00:00':data.training_date; 
      const trainingDateObj = new Date(training_date);
      trainingDateObj.setFullYear(trainingDateObj.getFullYear() + 2);
      const refresh_date = trainingDateObj.toISOString().split('T')[0];
	
      const emailStream = await createSshTunnel(sshTunnelConfig, dbServer);
      const emailCheckQuery = `SELECT email1 FROM membership WHERE email1 = '${per_email}'`;
      const existingUser = await queryDatabase(emailCheckQuery, emailStream);

      if (existingUser.length > 0) {
        return res.status(500).json({ message: 'Email already exists' });
      }
      else{
        const sqlQuery = `INSERT INTO membership (parent_id,title1,fname1,sname1,passport1,startdate,refresh_date,address1,address2,city1,province1,postcode1,country1,cellphone1,email1,verifyEmail,verifyAmount,choose_membership,membertype,status1,course_id,dive_location) VALUES ('${parent_id}','${per_title}','${first_name}','${surname}','${per_passport}','${training_date}','${refresh_date}','${per_address1}','${per_address2}','${per_city}','${per_province}','${post_code}','${per_country}','${cell_phone}','${per_email}','not_verified','not_paid','annual','Provider Student','INACTIVE','${course_type}','${course_loc}')`;  
        
        await queryDatabase(sqlQuery,stream);   	        

        // Send a success response
        res.status(200).json({ message: 'Successfully added students.' });   					  		
      }
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error });
  }
}); 

app.get('/providers/:id', async (req, res) => {    
  const id = req.params.id;		 
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);	  	
    const sqlQuery = `SELECT * FROM membership as p join courses as c ON c.id=p.course_id WHERE p.parent_id=${id} AND membertype='Provider Student' group by p.id`;	   	
    const results = await queryDatabase(sqlQuery, stream);												  							  		
    res.status(200).json(results);       			
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });		   			   	
  }
}) 

app.get('/instructor_courses', async (req, res) => {
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);		
    const sqlQuery = "SELECT * FROM courses where type='Instructor'";	          
    const results = await queryDatabase(sqlQuery, stream);	  			
    res.status(200).json(results);
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });		    	
  }
}); 

// Handle other routes
app.use((req, res) => {
  res.status(404).send('Not Found');   		
}); 

// Export the app 
module.exports = app;	   