const express = require('express');
const bodyParser = require('body-parser');  
const { sshTunnelConfig, dbServer } = require('./../config');
const createSshTunnel = require('./../sshTunnel');
const queryDatabase = require('./../databasequery');
const cors = require('cors');   

const app = express();
const PORT = 4500;

// Middleware to handle CORS and preflight requests
app.use((req, res, next) => {
    res.append('Access-Control-Allow-Origin', '*');
    res.append('Access-Control-Allow-Methods', 'GET,HEAD,PUT,PATCH,POST,DELETE,OPTIONS');
    res.append('Access-Control-Allow-Headers', 'Content-Type, authorization');
    next();
});

app.use(bodyParser.json());  
app.use(cors()); 

// Define routes for App1 
app.get('/', (req, res) => { 
	res.send('Hello from app28!');    					  				    
});  

app.get('/get_members/:id', async (req, res) => {    
  const id = req.params.id;		 				
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);	  	
    const sqlQuery = `SELECT id,fname1,sname1,member_no,email1 from membership WHERE parent_id=${id} AND UPPER(membertype)='INDUSTRY_PARTNER'`;	  	     	    
    const results = await queryDatabase(sqlQuery,stream);	    					       												  							  		
    res.status(200).json(results);   								
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });		  					    						   	
  }
}) 

app.get('/get_details/:id', async (req, res) => {    
  const id = req.params.id;		 				
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);	  	
    const sqlQuery = `SELECT * FROM membership WHERE UPPER(choose_membership)='PARTNER' AND parent_id=${id} AND instructor_id=${id}`;		     	    
    const results = await queryDatabase(sqlQuery,stream);	    					       												  							  		
    res.status(200).json(results);   										
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });		  					    						   	
  }
}) 

app.get('/edit/:id', async (req, res) => { 
  const id = req.params.id;		 
  try {
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);		
    const sqlQuery = `SELECT * FROM membership WHERE id=${id}`;	 			   				  		 		     	    
    const results = await queryDatabase(sqlQuery, stream);	  	     		
    res.status(200).json(results);          
  } catch (error) {
    res.status(500).json({ error: 'Error fetching data from database: ' + error.message });		      	
  }
})   	


app.post('/add', async (req, res) => {			
  const data = req.body;			
  
  const indusrtyEmail=(typeof data.email_address=="undefined" || data.email_address==null)?'':data.email_address;   
  const memberNo=(typeof data.member_no=="undefined" || data.member_no==null)?'DANIP':'DANIP '+data.member_no;  	    
  
  const stream = await createSshTunnel(sshTunnelConfig, dbServer);			     
  const sqlQuery = `SELECT * FROM membership WHERE email1='${indusrtyEmail}'`;	   			
  const chkresult = await queryDatabase(sqlQuery, stream);	  	

  const steam = await createSshTunnel(sshTunnelConfig,dbServer);			     
  const sqlQuer=`SELECT * FROM membership WHERE UPPER(choose_membership)='ANNUAL' AND UPPER(membertype)='INDUSTRY_PARTNER' AND member_no='${memberNo}'`;  			  			
  const chkrest = await queryDatabase(sqlQuer, steam);	 
  
  if (!data.title || data.title === "") {
    return res.status(500).json({ error: 'Title is Missing','error_field':'title' });		
  } 
  else if (!data.first_name || data.first_name === "") {
    return res.status(500).json({ error: 'First name is Missing','error_field':'first_name' });		
  }   
  else if (!data.dob || data.dob === "") {
    return res.status(500).json({ error: 'Date of birth is Missing','error_field':'dob' });					
  }   
  else if (!data.cell_phone || data.cell_phone === "") {
    return res.status(500).json({ error: 'Cell phone is Missing','error_field':'cell_phone' });					
  }
  else if (!data.email_address || data.email_address === "") {
    return res.status(500).json({ error: 'Email is Missing','error_field':'email_address' });		
  }     
  else if (chkresult.length >0) {		
    return res.status(500).json({ error: 'Email already exist','error_field':'email_address' });							
  } 
  else if (chkrest.length >0) {		
    return res.status(500).json({ error: 'Member No. already exist','error_field':'member_no' });									
  } 

  try {
		 // Establish SSH tunnel
		const stream = await createSshTunnel(sshTunnelConfig,dbServer); 	

		const parent_id=(typeof data.parent_id=="undefined")?'':data.parent_id;  	
		const title=(typeof data.title=="undefined" || data.title==null)?'':data.title;  	
		const fname=(typeof data.first_name=="undefined" || data.first_name==null)?'':data.first_name;    
		const sname=(typeof data.surname=="undefined" || data.surname==null)?'':data.surname;  	
		const dob=(typeof data.dob=="undefined" || data.dob== null)?'0000:00:00':data.dob;   
		const passport=(typeof data.passport=="undefined" || data.passport==null)?'':data.passport;  		
		const address1=(typeof data.address1=="undefined" || data.address1==null)?'':data.address1;  	
		const address2=(typeof data.address2=="undefined" || data.address2==null)?'':data.address2;  	
		const city=(typeof data.city=="undefined" || data.city==null)?'':data.city;  	
		const province=(typeof data.province=="undefined" || data.province==null)?'':data.province;   	
		const postal_code=(typeof data.post_code=="undefined" || data.post_code==null)?'':data.post_code;  	
		const country=(typeof data.country=="undefined" || data.country==null)?'':data.country;    
		const phonehome=(typeof data.tel_home=="undefined" || data.tel_home==null)?'':data.tel_home;  		
		const phonework=(typeof data.tel_work=="undefined" || data.tel_work==null)?'':data.tel_work;  
		const fax=(typeof data.fax=="undefined" || data.fax==null)?'':data.fax;  
		const cellphone=(typeof data.cell_phone=="undefined" || data.cell_phone==null)?'':data.cell_phone;  	
		const email_address=(typeof data.email_address=="undefined" || data.email_address==null)?'':data.email_address;  			 		
		const agency=(typeof data.agency=="undefined" || data.agency==null)?'':data.agency;  	
					
		const new_pwd = generateRandomPassword(Math.floor(Math.random() * 2) + 6); // Generate a random password with length between 6 and 7 characters
			
		const sqlQuery = `INSERT INTO membership (title1,fname1,sname1,dob1,passport1,address1,address2,city1,province1,postcode1,country1,telephone1,telephone2,cellphone1,email1,password,member_no,verifyEmail,verifyAmount,agency,fax,status1,parent_id,choose_membership,membertype) VALUES 
		('${title}','${fname}','${sname}','${dob}','${passport}','${address1}','${address2}','${city}','${province}','${postal_code}','${country}','${phonehome}','${phonework}','${cellphone}','${email_address}','${new_pwd}','${memberNo}','verified','not_paid','${agency}','${fax}','ACTIVE','${parent_id}','annual','industry_partner')`;  		
				
		const result = await queryDatabase(sqlQuery,stream);    																			   		  
		const insert_id = result.insertId;    		  
		
		if(insert_id >0)	
		{
			// Send a success response	 		
			res.status(200).json({ message: 'Data added successfully' });   														   																		  			 		  	
		} 
		else
		{
			res.status(500).json({ error: 'Record could not inserted!' });  												            
		} 	  
      
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);
    res.status(500).json({ error: error });
  }
}); 


app.post('/update/:id', async (req, res) => { 
  const id = req.params.id; 
  const data = req.body;  

  const stream = await createSshTunnel(sshTunnelConfig, dbServer);	     
  const sqlQuery = `SELECT * FROM membership WHERE id=${id}`;			 	    
  const result = await queryDatabase(sqlQuery,stream);	
  const emailId = (result.length>0)?result[0].email1:'';     
  const memberId = (result.length>0)?result[0].member_no:'';    
  
  const indusrtyEmail=(typeof data.email_address=="undefined" || data.email_address==null)?'':data.email_address;     
  const memberNo=(typeof data.member_no=="undefined" || data.member_no==null)?'':data.member_no;      
  
  const steam = await createSshTunnel(sshTunnelConfig, dbServer);			     
  const sqlQuer = `SELECT * FROM membership WHERE email1='${indusrtyEmail}'`;	   			
  const chkresult = await queryDatabase(sqlQuer,steam);	 

  const stem = await createSshTunnel(sshTunnelConfig,dbServer);			     
  const sqlQue=`SELECT * FROM membership WHERE member_no='${memberNo}'`;  			  			
  const chkrest = await queryDatabase(sqlQue,stem);	  
  
  if (!data.title || data.title === "") {
    return res.status(500).json({ error: 'Title is Missing','error_field':'title' });		
  } 
  else if (!data.first_name || data.first_name === "") {
    return res.status(500).json({ error: 'First name is Missing','error_field':'first_name' });		
  }   
  else if (!data.dob || data.dob === "") {
    return res.status(500).json({ error: 'Date of birth is Missing','error_field':'dob' });					
  }   
  else if (!data.cell_phone || data.cell_phone === "") {
    return res.status(500).json({ error: 'Cell phone is Missing','error_field':'cell_phone' });					
  }
  else if (!data.email_address || data.email_address === "") {
    return res.status(500).json({ error: 'Email is Missing','error_field':'email_address' });		
  }     
  else if (chkresult.length >0 && emailId !=indusrtyEmail) {		
    return res.status(500).json({ error: 'Email already exist','error_field':'email_address' });						
  }  
  else if (chkrest.length >0 && memberId !=memberNo) {		
    return res.status(500).json({ error: 'Member No. already exist','error_field':'member_no' });												
  }   
  
  if(result.length==1)  
  {
	  try {  

		// Establish SSH tunnel
        const stream = await createSshTunnel(sshTunnelConfig,dbServer);   				
		
		const title=(typeof data.title=="undefined" || data.title==null)?'':data.title;  	
		const fname=(typeof data.first_name=="undefined" || data.first_name==null)?'':data.first_name;    
		const sname=(typeof data.surname=="undefined" || data.surname==null)?'':data.surname;  	
		const dob=(typeof data.dob=="undefined" || data.dob== null)?'0000:00:00':data.dob;   
		const passport=(typeof data.passport=="undefined" || data.passport==null)?'':data.passport;  		
		const address1=(typeof data.address1=="undefined" || data.address1==null)?'':data.address1;  	
		const address2=(typeof data.address2=="undefined" || data.address2==null)?'':data.address2;  	
		const city=(typeof data.city=="undefined" || data.city==null)?'':data.city;  	
		const province=(typeof data.province=="undefined" || data.province==null)?'':data.province;   	
		const postal_code=(typeof data.post_code=="undefined" || data.post_code==null)?'':data.post_code;  	
		const country=(typeof data.country=="undefined" || data.country==null)?'':data.country;    
		const phonehome=(typeof data.tel_home=="undefined" || data.tel_home==null)?'':data.tel_home;  		
		const phonework=(typeof data.tel_work=="undefined" || data.tel_work==null)?'':data.tel_work;  
		const fax=(typeof data.fax=="undefined" || data.fax==null)?'':data.fax;  
		const cellphone=(typeof data.cell_phone=="undefined" || data.cell_phone==null)?'':data.cell_phone;  	
		const email_address=(typeof data.email_address=="undefined" || data.email_address==null)?'':data.email_address;  			 		
		const agency=(typeof data.agency=="undefined" || data.agency==null)?'':data.agency;  	 	 	 			
												
		const sqlQuery = `UPDATE membership SET title1='${title}',fname1='${fname}',sname1='${sname}',dob1='${dob}',passport1='${passport}',address1='${address1}',address2='${address2}',
		city1='${city}',province1='${province}',postcode1='${postal_code}',country1='${country}',telephone1='${phonehome}',telephone2='${phonework}',cellphone1='${cellphone}',email1='${email_address}',
		agency='${agency}',member_no='${memberNo}',fax='${fax}' WHERE id=${id}`;  		
		
		const result = await queryDatabase(sqlQuery,stream);  						
		
		if(id >0)	 
		{
			// Send a success response	  				
			res.status(200).json({ message: 'Record updated successfully' });  																		     					  			 		  	
		} 
		else
		{
			res.status(500).json({ error: 'Record could not updated!' });  	  				               
		}       	    		 		
		 
	  } catch (error) {
		// Handle database or other errors 	  		
		console.error('Error saving data in to the database:', error);	
		return res.status(500).json({ error: error });				
	  }
	  //
  }
  else
  {
	  return res.status(500).json({ error:'Record not found!' });	  				
  }  
  
})        

app.post('/save_detail/:id', async (req, res) => { 
  const id = req.params.id; 
  const data = req.body;   		  
  
  const stream = await createSshTunnel(sshTunnelConfig, dbServer);	     
  const sqlQuery = `SELECT * FROM membership WHERE UPPER(choose_membership)='PARTNER' AND parent_id=${id} AND instructor_id=${id}`;	
  
  const result = await queryDatabase(sqlQuery,stream);	
  const rowId = (result.length>0)?result[0].id:'';  	
  const emailId = (result.length>0)?result[0].email1:'';  	
  
  const partnerEmail=(typeof data.partner_email=="undefined" || data.partner_email==null)?'':data.partner_email;     
  
  const steam = await createSshTunnel(sshTunnelConfig,dbServer);			     
  const sqlQuer = `SELECT * FROM membership WHERE UPPER(choose_membership)='PARTNER' AND email1='${partnerEmail}'`;	   			
  const chkresult = await queryDatabase(sqlQuer,steam);	    										 

  if (!data.bname || data.bname === "") {
    return res.status(500).json({ error: 'Business name is required','error_field':'bname' });							
  }
  else if (!data.cname || data.cname === "") {
    return res.status(500).json({ error: 'Contact person name is required','error_field':'cname' });  
  }
  else if (!data.address3 || data.address3 === "") {
    return res.status(500).json({ error: 'Postal Address 1 is required','error_field':'address3' });  	
  }
  else if (!data.partner_city || data.partner_city === "") {
    return res.status(500).json({ error: 'City is required','error_field':'partner_city' }); 	 
  }   
  else if (!data.partner_province || data.partner_province === "") {	
    return res.status(500).json({ error: 'State/region/province is required','error_field':'partner_province' });  
  } 
  else if (!data.partner_zip || data.partner_zip === "") {
    return res.status(500).json({ error: 'Postal/Zip code is required','error_field':'partner_zip' });  	
  }
  else if (!data.partner_country || data.partner_country === "") {
    return res.status(500).json({ error: 'Country is required','error_field':'partner_country' });  
  } 
  else if (!data.workphone || data.workphone === "") {
    return res.status(500).json({ error: 'Telephone (W) is required','error_field':'workphone' });		
  } else if (!data.cellphone || data.cellphone === "") {
    return res.status(500).json({ error: 'Cell phone is required','error_field':'cellphone' });			
  }
  else if (!data.partner_email || data.partner_email === "") {
    return res.status(500).json({ error: 'Email address is required','error_field':'partner_email' }); 	 
  } 
  else if (chkresult.length >0 && emailId !=partnerEmail) {		
    return res.status(500).json({ error: 'Email already exist','error_field':'partner_email' });						
  }  	

  try {
    // Establish SSH tunnel
    const stream = await createSshTunnel(sshTunnelConfig, dbServer);	   

	const businessName=(typeof data.bname=="undefined")?'':data.bname;  	
	const contactName=(typeof data.cname=="undefined")?'':data.cname;  	
	const infoAddress1=(typeof data.address3=="undefined")?'':data.address3;    
	const infoAddress2=(typeof data.address4=="undefined")?'':data.address4;  	
	
	const infoCity=(typeof data.partner_city=="undefined")?'':data.partner_city;   	
	const infoProvince=(typeof data.partner_province=="undefined")?'':data.partner_province;  		
	const postCode=(typeof data.partner_zip=="undefined")?'':data.partner_zip;   
	const infoCountry=(typeof data.partner_country=="undefined")?'':data.partner_country;  	  	
	const telWork=(typeof data.workphone=="undefined")?'':data.workphone;    	
	const cellPhone=(typeof data.cellphone=="undefined")?'':data.cellphone;  	  
	const fax=(typeof data.partner_fax=="undefined")?'':data.partner_fax;  	
	
	const businesEmail=(typeof data.partner_email=="undefined")?'':data.partner_email;    
	const webAddress=(typeof data.waddress=="undefined")?'':data.waddress;   	          
    
	if(result.length>0) 
	{	
		const sqlQuery = `UPDATE membership SET company_name='${businessName}',contact_person='${contactName}',address1='${infoAddress1}',address2='${infoAddress2}',
		email1='${businesEmail}',address3='${webAddress}',city1='${infoCity}',province1='${infoProvince}',postcode1='${postCode}',country1='${infoCountry}',telephone1='${telWork}',
		cellphone1='${cellPhone}',fax='${fax}' WHERE id=${rowId}`;  
		await queryDatabase(sqlQuery,stream);   
		var insert_id = id;  	
		var msg1 = 'Data updated successfully';  				
		var msg2 = 'Record could not updated!'; 
	}
	else 
	{
		 const sqlQuery = `INSERT INTO membership (company_name,contact_person,address1,address2,email1,parent_id,instructor_id,address3,city1,province1,postcode1,country1,telephone1,cellphone1,fax,choose_membership) VALUES 
		('${businessName}','${contactName}','${infoAddress1}','${infoAddress2}','${businesEmail}','${id}','${id}','${webAddress}','${infoCity}','${infoProvince}','${postCode}','${infoCountry}','${telWork}','${cellPhone}','${fax}','partner')`;			
		
		const result = await queryDatabase(sqlQuery,stream);   		 		  
		var insert_id = result.insertId;  
		var msg1 = 'Data added successfully';  			
		var msg2 = 'Record could not inserted!';				
	}
    
	if(insert_id >0)  
	{
		// Send a success response	  								
		res.status(200).json({ message:`'${msg1}'` });     																			 		  	
	} 
	else
	{
		res.status(500).json({ error: `'${msg2}'` });  					   		         		
	} 	   		  		
	 
  } catch (error) {
    // Handle database or other errors 	  		
    console.error('Error adding data to the database:', error);		
    res.status(500).json({ error: error });
  }
    
})        

function generateRandomPassword(length) {
    const charset = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"; 		
    let password = "";
    for (let i = 0; i < length; i++) {
        const randomIndex = Math.floor(Math.random() * charset.length);
        password += charset[randomIndex];
    }
    return password;
}

// Handle other routes
app.use((req, res) => {
  res.status(404).send('Not Found');   		
}); 

// Export the app 
module.exports = app;	   